@@ -42,12 +42,14 @@
 #include <unistd.h>
 #include <sys/poll.h>
 #include <sys/wait.h>
+#include <sys/resource.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/syscall.h>
 #include <sys/utsname.h>
 #include <termios.h>
 #include <assert.h>
+#include <grp.h>
 
 #include <proc/readproc.h>
 #include <libudev.h>
@@ -630,8 +632,10 @@ static void igt_interactive_info(const char *format, ...)
 {
 	va_list args;
 
-	if (!isatty(STDERR_FILENO) || __igt_plain_output)
+	if (!isatty(STDERR_FILENO) || __igt_plain_output) {
+		errno = 0; /* otherwise would be either ENOTTY or EBADF */
 		return;
+	}
 
 	if (igt_log_level > IGT_LOG_INFO)
 		return;
@@ -682,63 +686,6 @@ void igt_print_activity(void)
 	igt_interactive_info(".");
 }
 
-/* mappable aperture trasher helper */
-drm_intel_bo **trash_bos;
-int num_trash_bos;
-
-/**
- * igt_init_aperture_trashers:
- * @bufmgr: libdrm buffer manager
- *
- * Initialize the aperture trasher using @bufmgr, which can then be run with
- * igt_trash_aperture().
- */
-void igt_init_aperture_trashers(drm_intel_bufmgr *bufmgr)
-{
-	int i;
-
-	num_trash_bos = gem_mappable_aperture_size() / (1024*1024);
-
-	trash_bos = malloc(num_trash_bos * sizeof(drm_intel_bo *));
-	igt_assert(trash_bos);
-
-	for (i = 0; i < num_trash_bos; i++)
-		trash_bos[i] = drm_intel_bo_alloc(bufmgr, "trash bo", 1024*1024, 4096);
-}
-
-/**
- * igt_trash_aperture:
- *
- * Trash the aperture by walking a set of GTT memory mapped objects.
- */
-void igt_trash_aperture(void)
-{
-	int i;
-	uint8_t *gtt_ptr;
-
-	for (i = 0; i < num_trash_bos; i++) {
-		drm_intel_gem_bo_map_gtt(trash_bos[i]);
-		gtt_ptr = trash_bos[i]->virtual;
-		*gtt_ptr = 0;
-		drm_intel_gem_bo_unmap_gtt(trash_bos[i]);
-	}
-}
-
-/**
- * igt_cleanup_aperture_trashers:
- *
- * Clean up all aperture trasher state set up with igt_init_aperture_trashers().
- */
-void igt_cleanup_aperture_trashers(void)
-{
-	int i;
-
-	for (i = 0; i < num_trash_bos; i++)
-		drm_intel_bo_unreference(trash_bos[i]);
-
-	free(trash_bos);
-}
-
 static int autoresume_delay;
 
 static const char *suspend_state_name[] = {
@@ -775,6 +722,11 @@ static enum igt_suspend_test get_suspend_test(int power_dir)
 			break;
 		}
 
+	if (!test_name) {
+	  	free(test_line);
+		return SUSPEND_TEST_NONE;
+	}
+
 	for (test = SUSPEND_TEST_NONE; test < SUSPEND_TEST_NUM; test++)
 		if (strcmp(suspend_test_name[test], test_name) == 0)
 			break;
@@ -782,7 +734,6 @@ static enum igt_suspend_test get_suspend_test(int power_dir)
 	igt_assert(test < SUSPEND_TEST_NUM);
 
 	free(test_line);
-
 	return test;
 }
 
@@ -1007,13 +958,15 @@ int igt_get_autoresume_delay(enum igt_suspend_state state)
  */
 void igt_drop_root(void)
 {
-	igt_assert(getuid() == 0);
+	igt_assert_eq(getuid(), 0);
 
-	igt_assert(setgid(2) == 0);
-	igt_assert(setuid(2) == 0);
+	igt_assert_eq(setgroups(0, NULL), 0);
+	igt_assert_eq(setgid(2), 0);
+	igt_assert_eq(setuid(2), 0);
 
-	igt_assert(getgid() == 2);
-	igt_assert(getuid() == 2);
+	igt_assert_eq(getgroups(0, NULL), 0);
+	igt_assert_eq(getgid(), 2);
+	igt_assert_eq(getuid(), 2);
 }
 
 /**
@@ -1036,8 +989,10 @@ void igt_debug_wait_for_keypress(const char *var)
 {
 	struct termios oldt, newt;
 
-	if (!isatty(STDIN_FILENO))
+	if (!isatty(STDIN_FILENO)) {
+		errno = 0; /* otherwise would be either ENOTTY or EBADF */
 		return;
+	}
 
 	if (!igt_interactive_debug)
 		return;
@@ -1082,8 +1037,10 @@ void igt_debug_manual_check(const char *var, const char *expected)
 	struct termios oldt, newt;
 	char key;
 
-	if (!isatty(STDIN_FILENO))
+	if (!isatty(STDIN_FILENO)) {
+		errno = 0; /* otherwise would be either ENOTTY or EBADF */
 		return;
+	}
 
 	if (!igt_interactive_debug)
 		return;
@@ -1137,14 +1094,14 @@ void igt_lock_mem(size_t size)
 
 	locked_mem = malloc(locked_size);
 	igt_require_f(locked_mem,
-		      "Could not allocate enough memory to lock.\n");
+		      "Could not malloc %zdMiB for locking.\n", size);
 
 	/* write into each page to ensure it is allocated */
 	for (i = 0; i < locked_size; i += pagesize)
 		locked_mem[i] = i;
 
 	ret = mlock(locked_mem, locked_size);
-	igt_assert_f(ret == 0, "Could not lock memory into RAM.\n");
+	igt_assert_f(ret == 0, "Could not mlock %zdMiB.\n", size);
 }
 
 /**
@@ -1305,6 +1262,37 @@ void igt_set_module_param_int(const char *name, int val)
 	igt_set_module_param(name, str);
 }
 
+/**
+ * igt_is_process_running:
+ * @comm: Name of process in the form found in /proc/pid/comm (limited to 15
+ * chars)
+ *
+ * Returns: true in case the process has been found, false otherwise.
+ *
+ * This function checks in the process table for an entry with the name @comm.
+ */
+int igt_is_process_running(const char *comm)
+{
+	PROCTAB *proc;
+	proc_t *proc_info;
+	bool found = false;
+
+	proc = openproc(PROC_FILLCOM | PROC_FILLSTAT);
+	igt_assert(proc != NULL);
+
+	while ((proc_info = readproc(proc, NULL))) {
+		if (!strncasecmp(proc_info->cmd, comm, sizeof(proc_info->cmd))) {
+			freeproc(proc_info);
+			found = true;
+			break;
+		}
+		freeproc(proc_info);
+	}
+
+	closeproc(proc);
+	return found;
+}
+
 /**
  * igt_terminate_process:
  * @sig: Signal to send
@@ -1473,6 +1461,8 @@ again:
 		free(copy_fd_lnk);
 		free(fd_lnk);
 	}
+
+	closedir(dp);
 }
 
 /*
@@ -1639,3 +1629,47 @@ double igt_stop_siglatency(struct igt_mean *result)
 
 	return mean;
 }
+
+bool igt_allow_unlimited_files(void)
+{
+	struct rlimit rlim;
+	unsigned nofile_rlim = 1024*1024;
+
+	FILE *file = fopen("/proc/sys/fs/nr_open", "r");
+	if (file) {
+		igt_assert(fscanf(file, "%u", &nofile_rlim) == 1);
+		igt_info("System limit for open files is %u\n", nofile_rlim);
+		fclose(file);
+	}
+
+	if (getrlimit(RLIMIT_NOFILE, &rlim))
+		return false;
+
+	rlim.rlim_cur = nofile_rlim;
+	rlim.rlim_max = nofile_rlim;
+	return setrlimit(RLIMIT_NOFILE, &rlim) == 0;
+}
+
+/**
+ * vfs_file_max: report maximum number of files
+ *
+ * Get the global system-wide maximum of open files the kernel allows,
+ * by reading /proc/sys/fs/file-max. Fails the current subtest if
+ * reading the file fails, and returns a suitable best guess if it
+ * cannot be opened.
+ *
+ * Returns: System-wide maximum of open files, or a best effort guess.
+ */
+uint64_t vfs_file_max(void)
+{
+	static long long unsigned max;
+	if (max == 0) {
+		FILE *file = fopen("/proc/sys/fs/file-max", "r");
+		max = 80000;
+		if (file) {
+			igt_assert(fscanf(file, "%llu", &max) == 1);
+			fclose(file);
+		}
+	}
+	return max;
+}
