@@ -73,6 +73,7 @@
 
 #define UNW_LOCAL_ONLY
 #include <libunwind.h>
+#include <elfutils/libdwfl.h>
 
 #ifdef HAVE_LIBGEN_H
 #include <libgen.h>   /* for basename() on Solaris */
@@ -213,10 +214,10 @@
  * code will only reflect the overall result.
  *
  * To do that obtain the lists of subtests with "--list-subtests", which can be
- * run as non-root and doesn't require the i915 driver to be loaded (or any
- * intel gpu to be present). Then individual subtests can be run with
- * "--run-subtest". Usage help for tests with subtests can be obtained with the
- * "--help" command line option.
+ * run as non-root and doesn't require a DRM driver to be loaded (or any GPU to
+ * be present). Then individual subtests can be run with "--run-subtest". Usage
+ * help for tests with subtests can be obtained with the "--help" command line
+ * option.
  *
  * A wildcard expression can be given to --run-subtest to specify a subset of
  * subtests to run. See https://tools.ietf.org/html/rfc3977#section-4 for a
@@ -235,7 +236,7 @@
  *
  * Some of IGT's behavior can be configured through a configuration file.
  * By default, this file is expected to exist in ~/.igtrc . The directory for
- * this can be overriden by setting the environment variable %IGT_CONFIG_PATH.
+ * this can be overridden by setting the environment variable %IGT_CONFIG_PATH.
  * An example configuration follows:
  *
  * |[<!-- language="plain" -->
@@ -279,12 +280,16 @@ int test_children_sz;
 bool test_child;
 
 enum {
- OPT_LIST_SUBTESTS,
- OPT_RUN_SUBTEST,
- OPT_DESCRIPTION,
- OPT_DEBUG,
- OPT_INTERACTIVE_DEBUG,
- OPT_HELP = 'h'
+	/*
+	 * Let the first values be used by individual tests so options don't
+	 * conflict with core ones
+	 */
+	OPT_LIST_SUBTESTS = 500,
+	OPT_RUN_SUBTEST,
+	OPT_DESCRIPTION,
+	OPT_DEBUG,
+	OPT_INTERACTIVE_DEBUG,
+	OPT_HELP = 'h'
 };
 
 static int igt_exitcode = IGT_EXIT_SUCCESS;
@@ -301,6 +306,8 @@ GKeyFile *igt_key_file;
 
 char *igt_frame_dump_path;
 
+static bool stderr_needs_sentinel = false;
+
 const char *igt_test_name(void)
 {
 	return command_str;
@@ -401,9 +408,8 @@ void igt_kmsg(const char *format, ...)
 
 #define time_valid(ts) ((ts)->tv_sec || (ts)->tv_nsec)
 
-static double
-time_elapsed(struct timespec *then,
-	     struct timespec* now)
+double igt_time_elapsed(struct timespec *then,
+			struct timespec *now)
 {
 	double elapsed = -1.;
 
@@ -415,7 +421,7 @@ time_elapsed(struct timespec *then,
 	return elapsed;
 }
 
-static int gettime(struct timespec *ts)
+int igt_gettime(struct timespec *ts)
 {
 	memset(ts, 0, sizeof(*ts));
 	errno = 0;
@@ -448,7 +454,7 @@ uint64_t igt_nsec_elapsed(struct timespec *start)
 {
 	struct timespec now;
 
-	gettime(&now);
+	igt_gettime(&now);
 	if ((start->tv_sec | start->tv_nsec) == 0) {
 		*start = now;
 		return 0;
@@ -461,6 +467,7 @@ uint64_t igt_nsec_elapsed(struct timespec *start)
 bool __igt_fixture(void)
 {
 	assert(!in_fixture);
+	assert(test_with_subtests);
 
 	if (igt_only_list_subtests())
 		return false;
@@ -551,7 +558,7 @@ static void print_usage(const char *help_str, bool output_on_stderr)
 		   "  --debug[=log-domain]\n"
 		   "  --interactive-debug[=domain]\n"
 		   "  --help-description\n"
-		   "  --help\n");
+		   "  --help|-h\n");
 	if (help_str)
 		fprintf(f, "%s\n", help_str);
 }
@@ -589,7 +596,7 @@ static void common_init_config(void)
 	igt_key_file = g_key_file_new();
 	ret = g_key_file_load_from_file(igt_key_file, key_file_loc,
 					G_KEY_FILE_NONE, &error);
-	if (error && error->code == G_KEY_FILE_ERROR) {
+	if (!ret) {
 		g_error_free(error);
 		g_key_file_free(igt_key_file);
 		igt_key_file = NULL;
@@ -627,6 +634,8 @@ static void common_init_env(void)
 	if (!isatty(STDOUT_FILENO) || getenv("IGT_PLAIN_OUTPUT"))
 		__igt_plain_output = true;
 
+	errno = 0; /* otherwise may be either ENOTTY or EBADF because of isatty */
+
 	if (!__igt_plain_output)
 		setlocale(LC_ALL, "");
 
@@ -643,6 +652,13 @@ static void common_init_env(void)
 	}
 
 	igt_frame_dump_path = getenv("IGT_FRAME_DUMP_PATH");
+
+	stderr_needs_sentinel = getenv("IGT_SENTINEL_ON_STDERR") != NULL;
+
+	env = getenv("IGT_FORCE_DRIVER");
+	if (env) {
+		__set_forced_driver(env);
+	}
 }
 
 static int common_init(int *argc, char **argv,
@@ -654,16 +670,17 @@ static int common_init(int *argc, char **argv,
 {
 	int c, option_index = 0, i, x;
 	static struct option long_options[] = {
-		{"list-subtests", 0, 0, OPT_LIST_SUBTESTS},
-		{"run-subtest", 1, 0, OPT_RUN_SUBTEST},
-		{"help-description", 0, 0, OPT_DESCRIPTION},
-		{"debug", optional_argument, 0, OPT_DEBUG},
-		{"interactive-debug", optional_argument, 0, OPT_INTERACTIVE_DEBUG},
-		{"help", 0, 0, OPT_HELP},
+		{"list-subtests",     no_argument,       NULL, OPT_LIST_SUBTESTS},
+		{"run-subtest",       required_argument, NULL, OPT_RUN_SUBTEST},
+		{"help-description",  no_argument,       NULL, OPT_DESCRIPTION},
+		{"debug",             optional_argument, NULL, OPT_DEBUG},
+		{"interactive-debug", optional_argument, NULL, OPT_INTERACTIVE_DEBUG},
+		{"help",              no_argument,       NULL, OPT_HELP},
 		{0, 0, 0, 0}
 	};
 	char *short_opts;
 	const char *std_short_opts = "h";
+	size_t std_short_opts_len = strlen(std_short_opts);
 	struct option *combined_opts;
 	int extra_opt_count;
 	int all_opt_count;
@@ -675,52 +692,63 @@ static int common_init(int *argc, char **argv,
 	if (strrchr(command_str, '/'))
 		command_str = strrchr(command_str, '/') + 1;
 
-	/* First calculate space for all passed-in extra long options */
-	all_opt_count = 0;
-	while (extra_long_opts && extra_long_opts[all_opt_count].name) {
+	/* Check for conflicts and calculate space for passed-in extra long options */
+	for  (extra_opt_count = 0; extra_long_opts && extra_long_opts[extra_opt_count].name; extra_opt_count++) {
+		char *conflicting_char;
 
 		/* check for conflicts with standard long option values */
-		for (i = 0; long_options[i].name; i++)
-			if (extra_long_opts[all_opt_count].val == long_options[i].val)
-				igt_warn("Conflicting long option values between --%s and --%s\n",
-					 extra_long_opts[all_opt_count].name,
-					 long_options[i].name);
+		for (i = 0; long_options[i].name; i++) {
+			if (0 == strcmp(extra_long_opts[extra_opt_count].name, long_options[i].name)) {
+				igt_critical("Conflicting extra long option defined --%s\n", long_options[i].name);
+				assert(0);
 
-		/* check for conflicts with short options */
-		if (extra_long_opts[all_opt_count].val != ':'
-		    && strchr(std_short_opts, extra_long_opts[all_opt_count].val)) {
-			igt_warn("Conflicting long and short option values between --%s and -%s\n",
-				 extra_long_opts[all_opt_count].name,
-				 long_options[i].name);
+			}
+
+			if (extra_long_opts[extra_opt_count].val == long_options[i].val) {
+				igt_critical("Conflicting long option 'val' representation between --%s and --%s\n",
+					     extra_long_opts[extra_opt_count].name,
+					     long_options[i].name);
+				assert(0);
+			}
 		}
 
-
-		all_opt_count++;
+		/* check for conflicts with standard short options */
+		if (extra_long_opts[extra_opt_count].val != ':'
+		    && (conflicting_char = memchr(std_short_opts, extra_long_opts[extra_opt_count].val, std_short_opts_len))) {
+			igt_critical("Conflicting long and short option 'val' representation between --%s and -%c\n",
+				     extra_long_opts[extra_opt_count].name,
+				     *conflicting_char);
+			assert(0);
+		}
 	}
-	extra_opt_count = all_opt_count;
 
 	/* check for conflicts in extra short options*/
 	for (i = 0; extra_short_opts && extra_short_opts[i]; i++) {
-
 		if (extra_short_opts[i] == ':')
 			continue;
 
 		/* check for conflicts with standard short options */
-		if (strchr(std_short_opts, extra_short_opts[i]))
-			igt_warn("Conflicting short option: -%c\n", std_short_opts[i]);
+		if (memchr(std_short_opts, extra_short_opts[i], std_short_opts_len)) {
+			igt_critical("Conflicting short option: -%c\n", std_short_opts[i]);
+			assert(0);
+		}
 
 		/* check for conflicts with standard long option values */
-		for (x = 0; long_options[x].name; x++)
-			if (long_options[x].val == extra_short_opts[i])
-				igt_warn("Conflicting short option and long option value: --%s and -%c\n",
-					 long_options[x].name, extra_short_opts[i]);
+		for (x = 0; long_options[x].name; x++) {
+			if (long_options[x].val == extra_short_opts[i]) {
+				igt_critical("Conflicting short option and long option 'val' representation: --%s and -%c\n",
+					     long_options[x].name, extra_short_opts[i]);
+				assert(0);
+			}
+		}
 	}
 
-	all_opt_count += ARRAY_SIZE(long_options);
+	all_opt_count = extra_opt_count + ARRAY_SIZE(long_options);
 
 	combined_opts = malloc(all_opt_count * sizeof(*combined_opts));
-	memcpy(combined_opts, extra_long_opts,
-	       extra_opt_count * sizeof(*combined_opts));
+	if (extra_opt_count > 0)
+		memcpy(combined_opts, extra_long_opts,
+		       extra_opt_count * sizeof(*combined_opts));
 
 	/* Copy the subtest long options (and the final NULL entry) */
 	memcpy(&combined_opts[extra_opt_count], long_options,
@@ -750,6 +778,7 @@ static int common_init(int *argc, char **argv,
 				list_subtests = true;
 			break;
 		case OPT_RUN_SUBTEST:
+			assert(optarg);
 			if (!list_subtests)
 				run_single_subtest = strdup(optarg);
 			break;
@@ -807,7 +836,7 @@ out:
 	igt_install_exit_handler(common_exit_handler);
 
 	if (!test_with_subtests)
-		gettime(&subtest_time);
+		igt_gettime(&subtest_time);
 
 	for (i = 0; (optind + i) < *argc; i++)
 		argv[i + 1] = argv[optind + i];
@@ -833,8 +862,18 @@ out:
  * additional knobs to tune when run manually like the number of rounds execute
  * or the size of the allocated buffer objects.
  *
- * Tests without special needs should just use igt_subtest_init() or use
- * #igt_main directly instead of their own main() function.
+ * Tests should use #igt_main_args instead of their own main()
+ * function and calling this function.
+ *
+ * The @help_str parameter is printed directly after the help text of
+ * standard arguments. The formatting of the string should be:
+ * - One line per option
+ * - Two spaces, option flag, tab character, help text, newline character
+ *
+ * Example: "  -s\tBuffer size\n"
+ *
+ * The opt handler function must return #IGT_OPT_HANDLER_SUCCESS on
+ * successful handling, #IGT_OPT_HANDLER_ERROR on errors.
  *
  * Returns: Forwards any option parsing errors from getopt_long.
  */
@@ -867,7 +906,22 @@ enum igt_log_level igt_log_level = IGT_LOG_INFO;
  * @handler_data: user data given to @extra_opt_handler when invoked
  *
  * This initializes a simple test without any support for subtests and allows
- * an arbitrary set of additional options.
+ * an arbitrary set of additional options. This is useful for tests which have
+ * additional knobs to tune when run manually like the number of rounds execute
+ * or the size of the allocated buffer objects.
+ *
+ * Tests should use #igt_simple_main_args instead of their own main()
+ * function and calling this function.
+ *
+ * The @help_str parameter is printed directly after the help text of
+ * standard arguments. The formatting of the string should be:
+ * - One line per option
+ * - Two spaces, option flag, tab character, help text, newline character
+ *
+ * Example: "  -s\tBuffer size\n"
+ *
+ * The opt handler function must return #IGT_OPT_HANDLER_SUCCESS on
+ * successful handling, #IGT_OPT_HANDLER_ERROR on errors.
  */
 void igt_simple_init_parse_opts(int *argc, char **argv,
 				const char *extra_short_opts,
@@ -889,9 +943,7 @@ bool __igt_run_subtest(const char *subtest_name)
 {
 	int i;
 
-	assert(!in_subtest);
-	assert(!in_fixture);
-	assert(test_with_subtests);
+	assert(!igt_can_fail());
 
 	/* check the subtest name only contains a-z, A-Z, 0-9, '-' and '_' */
 	for (i = 0; subtest_name[i] != '\0'; i++)
@@ -919,16 +971,24 @@ bool __igt_run_subtest(const char *subtest_name)
 		       (!__igt_plain_output) ? "\x1b[1m" : "", subtest_name,
 		       skip_subtests_henceforth == SKIP ?
 		       "SKIP" : "FAIL", (!__igt_plain_output) ? "\x1b[0m" : "");
+		fflush(stdout);
+		if (stderr_needs_sentinel)
+			fprintf(stderr, "Subtest %s: %s\n", subtest_name,
+				skip_subtests_henceforth == SKIP ?
+				"SKIP" : "FAIL");
 		return false;
 	}
 
 	igt_kmsg(KMSG_INFO "%s: starting subtest %s\n",
 		 command_str, subtest_name);
-	igt_debug("Starting subtest: %s\n", subtest_name);
+	igt_info("Starting subtest: %s\n", subtest_name);
+	fflush(stdout);
+	if (stderr_needs_sentinel)
+		fprintf(stderr, "Starting subtest: %s\n", subtest_name);
 
 	_igt_log_buffer_reset();
 
-	gettime(&subtest_time);
+	igt_gettime(&subtest_time);
 	return (in_subtest = subtest_name);
 }
 
@@ -956,6 +1016,8 @@ bool igt_only_list_subtests(void)
 
 void __igt_subtest_group_save(int *save)
 {
+	assert(test_with_subtests);
+
 	*save = skip_subtests_henceforth;
 }
 
@@ -973,14 +1035,17 @@ static void exit_subtest(const char *result)
 {
 	struct timespec now;
 
-	gettime(&now);
+	igt_gettime(&now);
 	igt_info("%sSubtest %s: %s (%.3fs)%s\n",
 		 (!__igt_plain_output) ? "\x1b[1m" : "",
-		 in_subtest, result, time_elapsed(&subtest_time, &now),
+		 in_subtest, result, igt_time_elapsed(&subtest_time, &now),
 		 (!__igt_plain_output) ? "\x1b[0m" : "");
 	fflush(stdout);
+	if (stderr_needs_sentinel)
+		fprintf(stderr, "Subtest %s: %s (%.3fs)\n",
+			in_subtest, result, igt_time_elapsed(&subtest_time, &now));
 
-	igt_terminate_spin_batches();
+	igt_terminate_spins();
 
 	in_subtest = NULL;
 	siglongjmp(igt_subtest_jmpbuf, 1);
@@ -1113,10 +1178,7 @@ void igt_fail(int exitcode)
 	_igt_log_buffer_dump();
 
 	if (in_subtest) {
-		if (exitcode == IGT_EXIT_TIMEOUT)
-			exit_subtest("TIMEOUT");
-		else
-			exit_subtest("FAIL");
+		exit_subtest("FAIL");
 	} else {
 		assert(igt_can_fail());
 
@@ -1198,20 +1260,59 @@ static void print_backtrace(void)
 	unw_context_t uc;
 	int stack_num = 0;
 
+	Dwfl_Callbacks cbs = {
+		.find_elf = dwfl_linux_proc_find_elf,
+		.find_debuginfo = dwfl_standard_find_debuginfo,
+	};
+
+	Dwfl *dwfl = dwfl_begin(&cbs);
+
+	if (dwfl_linux_proc_report(dwfl, getpid())) {
+		dwfl_end(dwfl);
+		dwfl = NULL;
+	} else
+		dwfl_report_end(dwfl, NULL, NULL);
+
 	igt_info("Stack trace:\n");
 
 	unw_getcontext(&uc);
 	unw_init_local(&cursor, &uc);
 	while (unw_step(&cursor) > 0) {
 		char name[255];
-		unw_word_t off;
+		unw_word_t off, ip;
+		Dwfl_Module *mod = NULL;
+
+		unw_get_reg(&cursor, UNW_REG_IP, &ip);
+
+		if (dwfl)
+			mod = dwfl_addrmodule(dwfl, ip);
+
+		if (mod) {
+			const char *src, *dwfl_name;
+			Dwfl_Line *line;
+			int lineno;
+			GElf_Sym sym;
+
+			line = dwfl_module_getsrc(mod, ip);
+			dwfl_name = dwfl_module_addrsym(mod, ip, &sym, NULL);
+
+			if (line && dwfl_name) {
+				src = dwfl_lineinfo(line, NULL, &lineno, NULL, NULL, NULL);
+				igt_info("  #%d %s:%d %s()\n", stack_num++, src, lineno, dwfl_name);
+				continue;
+			}
+		}
 
 		if (unw_get_proc_name(&cursor, name, 255, &off) < 0)
-			strcpy(name, "<unknown>");
-
-		igt_info("  #%d [%s+0x%x]\n", stack_num++, name,
-			 (unsigned int) off);
+			igt_info("  #%d [<unknown>+0x%x]\n", stack_num++,
+				 (unsigned int) ip);
+		else
+			igt_info("  #%d [%s+0x%x]\n", stack_num++, name,
+				 (unsigned int) off);
 	}
+
+	if (dwfl)
+		dwfl_end(dwfl);
 }
 
 static const char hex[] = "0123456789abcdef";
@@ -1431,6 +1532,8 @@ void __igt_fail_assert(const char *domain, const char *file, const int line,
  */
 void igt_exit(void)
 {
+	int tmp;
+
 	igt_exit_called = true;
 
 	if (igt_key_file)
@@ -1464,20 +1567,20 @@ void igt_exit(void)
 
 	for (int c = 0; c < num_test_children; c++)
 		kill(test_children[c], SIGKILL);
+	assert(!num_test_children);
+
+	assert(waitpid(-1, &tmp, WNOHANG) == -1 && errno == ECHILD);
 
 	if (!test_with_subtests) {
 		struct timespec now;
 		const char *result;
 
-		gettime(&now);
+		igt_gettime(&now);
 
 		switch (igt_exitcode) {
 			case IGT_EXIT_SUCCESS:
 				result = "SUCCESS";
 				break;
-			case IGT_EXIT_TIMEOUT:
-				result = "TIMEOUT";
-				break;
 			case IGT_EXIT_SKIP:
 				result = "SKIP";
 				break;
@@ -1486,7 +1589,7 @@ void igt_exit(void)
 		}
 
 		printf("%s (%.3fs)\n",
-		       result, time_elapsed(&subtest_time, &now));
+		       result, igt_time_elapsed(&subtest_time, &now));
 	}
 
 	exit(igt_exitcode);
@@ -1610,13 +1713,17 @@ static bool helper_was_alive(struct igt_helper_process *proc,
  * igt_stop_helper:
  * @proc: #igt_helper_process structure
  *
- * Terminates a helper process. It is an error to call this on a helper process
- * which hasn't been spawned yet.
+ * Terminates a helper process. It is legal to call this on a helper process
+ * which hasn't been spawned yet, e.g. if the helper was skipped due to
+ * HW restrictions.
  */
 void igt_stop_helper(struct igt_helper_process *proc)
 {
 	int status;
 
+	if (!proc->running) /* never even started */
+		return;
+
 	/* failure here means the pid is already dead and so waiting is safe */
 	kill(proc->pid, proc->use_SIGKILL ? SIGKILL : SIGTERM);
 
@@ -1664,6 +1771,7 @@ bool __igt_fork(void)
 		exit_handler_count = 0;
 		reset_helper_process_list();
 		oom_adjust_for_doom();
+		igt_unshare_spins();
 
 		return true;
 	default:
@@ -1672,32 +1780,6 @@ bool __igt_fork(void)
 
 }
 
-/**
- * igt_child_done:
- *
- * Lets the IGT core know that one of the children has exited.
- */
-void igt_child_done(pid_t pid)
-{
-	int i = 0;
-	int found = -1;
-
-	igt_assert(num_test_children > 1);
-
-	for (i = 0; i < num_test_children; i++) {
-		if (pid == test_children[i]) {
-			found = i;
-			break;
-		}
-	}
-
-	igt_assert(found >= 0);
-
-	num_test_children--;
-	for (i = found; i < num_test_children; i++)
-		test_children[i] = test_children[i + 1];
-}
-
 int __igt_waitchildren(void)
 {
 	int err = 0;
@@ -1767,20 +1849,40 @@ void igt_waitchildren(void)
 		igt_fail(err);
 }
 
+static void igt_alarm_killchildren(int signal)
+{
+	igt_info("Timed out waiting for children\n");
+
+	for (int c = 0; c < num_test_children; c++)
+		kill(test_children[c], SIGKILL);
+}
+
 /**
  * igt_waitchildren_timeout:
  * @seconds: timeout in seconds to wait
  * @reason: debug string explaining what timedout
  *
- * Wait for all children forked with igt_fork, for a maximum of @seconds.
- *
- * Wraps igt_waitchildren() and igt_set_timeout()
+ * Wait for all children forked with igt_fork, for a maximum of @seconds. If the
+ * timeout expires, kills all children, cleans them up, and then fails by
+ * calling igt_fail().
  */
 void igt_waitchildren_timeout(int seconds, const char *reason)
 {
-	igt_set_timeout(seconds, reason);
-	igt_waitchildren();
+	struct sigaction sa;
+	int ret;
+
+	sa.sa_handler = igt_alarm_killchildren;
+	sigemptyset(&sa.sa_mask);
+	sa.sa_flags = 0;
+
+	sigaction(SIGALRM, &sa, NULL);
+
+	alarm(seconds);
+
+	ret = __igt_waitchildren();
 	igt_reset_timeout();
+	if (ret)
+		igt_fail(ret);
 }
 
 /* exit handler code */
@@ -1798,14 +1900,29 @@ static struct {
 
 static igt_exit_handler_t exit_handler_fn[MAX_EXIT_HANDLERS];
 static bool exit_handler_disabled;
+static const struct {
+	int number;
+	const char *name;
+	size_t name_len;
+} handled_signals[] = {
 #define SIGDEF(x) { x, #x, sizeof(#x) - 1 }
 #define SILENT(x) { x, NULL, 0 }
-static const struct { int number; const char *name; size_t name_len; } handled_signals[] =
-	{ SILENT(SIGINT), SILENT(SIGHUP), SILENT(SIGTERM), SILENT(SIGQUIT),
-	  SILENT(SIGPIPE), SIGDEF(SIGABRT), SIGDEF(SIGSEGV), SIGDEF(SIGBUS),
-	  SIGDEF(SIGFPE) };
+
+	SILENT(SIGINT),
+	SILENT(SIGHUP),
+	SILENT(SIGPIPE),
+	SILENT(SIGTERM),
+
+	SIGDEF(SIGQUIT), /* used by igt_runner for its external timeout */
+
+	SIGDEF(SIGABRT),
+	SIGDEF(SIGSEGV),
+	SIGDEF(SIGBUS),
+	SIGDEF(SIGFPE)
+
 #undef SILENT
 #undef SIGDEF
+};
 
 static int install_sig_handler(int sig_num, sighandler_t handler)
 {
@@ -1837,7 +1954,7 @@ static void call_exit_handlers(int sig)
 {
 	int i;
 
-	igt_terminate_spin_batches();
+	igt_terminate_spins();
 
 	if (!exit_handler_count) {
 		return;
@@ -1886,15 +2003,15 @@ static void fatal_sig_handler(int sig)
 			__write_stderr(handled_signals[i].name,
 				       handled_signals[i].name_len);
 			write_stderr(".\n");
+
+			print_backtrace_sig_safe();
 		}
 
 		if (crash_signal(sig)) {
 			/* Linux standard to return exit code as 128 + signal */
 			if (!failed_one)
 				igt_exitcode = 128 + sig;
-
 			failed_one = true;
-			print_backtrace_sig_safe();
 
 			if (in_subtest)
 				exit_subtest("CRASH");
@@ -1914,7 +2031,7 @@ static void fatal_sig_handler(int sig)
 #ifdef __linux__
 	/* Workaround cached PID and TID races on glibc and Bionic libc. */
 		pid_t pid = syscall(SYS_getpid);
-		pid_t tid = syscall(SYS_gettid);
+		pid_t tid = gettid();
 
 		syscall(SYS_tgkill, pid, tid, sig);
 #else
@@ -2315,7 +2432,7 @@ err:
  */
 int igt_system_quiet(const char *command)
 {
-	int stderr_fd_copy, stdout_fd_copy, status, nullfd;
+	int stderr_fd_copy = -1, stdout_fd_copy = -1, status, nullfd = -1;
 
 	/* redirect */
 	if ((nullfd = open("/dev/null", O_WRONLY)) == -1)
