@@ -35,6 +35,7 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <i915_drm.h>
+#include <poll.h>
 
 #include "drmtest.h"
 #include "igt_aux.h"
@@ -235,6 +236,33 @@ int igt_debugfs_dir(int device)
 	return open(path, O_RDONLY);
 }
 
+/**
+ * igt_debugfs_connector_dir:
+ * @device: fd of the device
+ * @conn_name: conenctor name
+ * @mode: mode bits as used by open()
+ *
+ * This opens the debugfs directory corresponding to connector on the device
+ * for use with igt_sysfs_get() and related functions.
+ *
+ * Returns:
+ * The directory fd, or -1 on failure.
+ */
+int igt_debugfs_connector_dir(int device, char *conn_name, int mode)
+{
+	int dir, ret;
+
+	dir = igt_debugfs_dir(device);
+	if (dir < 0)
+		return dir;
+
+	ret = openat(dir, conn_name, mode);
+
+	close(dir);
+
+	return ret;
+}
+
 /**
  * igt_debugfs_open:
  * @filename: name of the debugfs node to open
@@ -261,26 +289,47 @@ int igt_debugfs_open(int device, const char *filename, int mode)
 	return ret;
 }
 
+/**
+ * igt_debugfs_simple_read:
+ * @filename: file name
+ * @buf: buffer where the contents will be stored, allocated by the caller
+ * @size: size of the buffer
+ *
+ * This function is similar to __igt_debugfs_read, the difference is that it
+ * expects the debugfs directory to be open and it's descriptor passed as the
+ * first argument.
+ *
+ * Returns:
+ * -errorno on failure or bytes read on success
+ */
+int igt_debugfs_simple_read(int dir, const char *filename, char *buf, int size)
+{
+	int len;
+
+	len = igt_sysfs_read(dir, filename, buf, size - 1);
+	if (len < 0)
+		buf[0] = '\0';
+	else
+		buf[len] = '\0';
+
+	return len;
+}
+
 /**
  * __igt_debugfs_read:
  * @filename: file name
  * @buf: buffer where the contents will be stored, allocated by the caller
- * @buf_size: size of the buffer
+ * @size: size of the buffer
  *
  * This function opens the debugfs file, reads it, stores the content in the
  * provided buffer, then closes the file. Users should make sure that the buffer
  * provided is big enough to fit the whole file, plus one byte.
  */
-void __igt_debugfs_read(int fd, const char *filename, char *buf, int buf_size)
+void __igt_debugfs_read(int fd, const char *filename, char *buf, int size)
 {
-	int dir;
-	int len;
+	int dir = igt_debugfs_dir(fd);
 
-	dir = igt_debugfs_dir(fd);
-	len = igt_sysfs_read(dir, filename, buf, buf_size - 1);
-	if (len < 0)
-		len = 0;
-	buf[len] = '\0';
+	igt_debugfs_simple_read(dir, filename, buf, size);
 	close(dir);
 }
 
@@ -409,17 +458,17 @@ char *igt_crc_to_string_extended(igt_crc_t *crc, char delimiter, int crc_size)
 	int i;
 	int len = 0;
 	int field_width = 2 * crc_size; /* Two chars per byte. */
-	char *buf = malloc((field_width+1) * crc->n_words * sizeof(char));
+	char *buf = malloc((field_width+1) * crc->n_words);
 
 	if (!buf)
 		return NULL;
 
-	for (i = 0; i < crc->n_words; i++)
+	for (i = 0; i < crc->n_words - 1; i++)
 		len += sprintf(buf + len, "%0*x%c", field_width,
 			       crc->crc[i], delimiter);
 
-	/* Eat the last delimiter */
-	buf[len - 1] = '\0';
+	sprintf(buf + len, "%0*x", field_width, crc->crc[i]);
+
 	return buf;
 }
 
@@ -448,27 +497,9 @@ struct _igt_pipe_crc {
 	int flags;
 
 	enum pipe pipe;
-	enum intel_pipe_crc_source source;
+	char *source;
 };
 
-static const char *pipe_crc_sources[] = {
-	"none",
-	"plane1",
-	"plane2",
-	"pf",
-	"pipe",
-	"TV",
-	"DP-B",
-	"DP-C",
-	"DP-D",
-	"auto"
-};
-
-static const char *pipe_crc_source_name(enum intel_pipe_crc_source source)
-{
-        return pipe_crc_sources[source];
-}
-
 /**
  * igt_require_pipe_crc:
  *
@@ -579,7 +610,7 @@ bool igt_hpd_storm_detected(int drm_fd)
 	if (fd < 0)
 		return false;
 
-	igt_assert_lt(0, read(fd, buf, sizeof(buf)));
+	igt_assert_lt(0, read(fd, buf, sizeof(buf) - 1));
 	igt_assert(start_loc = strstr(buf, "Detected: "));
 	igt_assert_eq(sscanf(start_loc, "Detected: %s\n", detected_str), 1);
 
@@ -611,12 +642,14 @@ void igt_require_hpd_storm_ctl(int drm_fd)
 }
 
 static igt_pipe_crc_t *
-pipe_crc_new(int fd, enum pipe pipe, enum intel_pipe_crc_source source, int flags)
+pipe_crc_new(int fd, enum pipe pipe, const char *source, int flags)
 {
 	igt_pipe_crc_t *pipe_crc;
 	char buf[128];
 	int debugfs;
 
+	igt_assert(source);
+
 	debugfs = igt_debugfs_dir(fd);
 	igt_assert(debugfs != -1);
 
@@ -630,7 +663,8 @@ pipe_crc_new(int fd, enum pipe pipe, enum intel_pipe_crc_source source, int flag
 	pipe_crc->fd = fd;
 	pipe_crc->dir = debugfs;
 	pipe_crc->pipe = pipe;
-	pipe_crc->source = source;
+	pipe_crc->source = strdup(source);
+	igt_assert(pipe_crc->source);
 	pipe_crc->flags = flags;
 
 	return pipe_crc;
@@ -649,7 +683,7 @@ pipe_crc_new(int fd, enum pipe pipe, enum intel_pipe_crc_source source, int flag
  * least INTEL_PIPE_CRC_SOURCE_AUTO everywhere.
  */
 igt_pipe_crc_t *
-igt_pipe_crc_new(int fd, enum pipe pipe, enum intel_pipe_crc_source source)
+igt_pipe_crc_new(int fd, enum pipe pipe, const char *source)
 {
 	return pipe_crc_new(fd, pipe, source, O_RDONLY);
 }
@@ -667,7 +701,7 @@ igt_pipe_crc_new(int fd, enum pipe pipe, enum intel_pipe_crc_source source)
  * least INTEL_PIPE_CRC_SOURCE_AUTO everywhere.
  */
 igt_pipe_crc_t *
-igt_pipe_crc_new_nonblock(int fd, enum pipe pipe, enum intel_pipe_crc_source source)
+igt_pipe_crc_new_nonblock(int fd, enum pipe pipe, const char *source)
 {
 	return pipe_crc_new(fd, pipe, source, O_RDONLY | O_NONBLOCK);
 }
@@ -686,6 +720,7 @@ void igt_pipe_crc_free(igt_pipe_crc_t *pipe_crc)
 	close(pipe_crc->ctl_fd);
 	close(pipe_crc->crc_fd);
 	close(pipe_crc->dir);
+	free(pipe_crc->source);
 	free(pipe_crc);
 }
 
@@ -735,8 +770,13 @@ static void read_one_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out)
 {
 	int ret;
 
-	while ((ret = read_crc(pipe_crc, out)) <= 0 && ret != -EINVAL)
-		usleep(1000);
+	fcntl(pipe_crc->crc_fd, F_SETFL, pipe_crc->flags & ~O_NONBLOCK);
+
+	do {
+		ret = read_crc(pipe_crc, out);
+	} while (ret == -EINTR);
+
+	fcntl(pipe_crc->crc_fd, F_SETFL, pipe_crc->flags);
 }
 
 /**
@@ -747,21 +787,29 @@ static void read_one_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out)
  */
 void igt_pipe_crc_start(igt_pipe_crc_t *pipe_crc)
 {
-	const char *src = pipe_crc_source_name(pipe_crc->source);
+	const char *src = pipe_crc->source;
+	struct pollfd pfd;
 	char buf[32];
 
 	/* Stop first just to make sure we don't have lingering state left. */
 	igt_pipe_crc_stop(pipe_crc);
 
+	igt_reset_fifo_underrun_reporting(pipe_crc->fd);
+
 	igt_assert_eq(write(pipe_crc->ctl_fd, src, strlen(src)), strlen(src));
 
 	sprintf(buf, "crtc-%d/crc/data", pipe_crc->pipe);
 
-	igt_set_timeout(10, "Opening crc fd, which waits for first CRC.");
+	igt_set_timeout(10, "Opening crc fd, and poll for first CRC.");
 	pipe_crc->crc_fd = openat(pipe_crc->dir, buf, pipe_crc->flags);
+	igt_assert(pipe_crc->crc_fd != -1);
+
+	pfd.fd = pipe_crc->crc_fd;
+	pfd.events = POLLIN;
+	poll(&pfd, 1, -1);
+
 	igt_reset_timeout();
 
-	igt_assert(pipe_crc->crc_fd != -1);
 	errno = 0;
 }
 
@@ -824,11 +872,15 @@ igt_pipe_crc_get_crcs(igt_pipe_crc_t *pipe_crc, int n_crcs,
 	return n;
 }
 
-static void crc_sanity_checks(igt_crc_t *crc)
+static void crc_sanity_checks(igt_pipe_crc_t *pipe_crc, igt_crc_t *crc)
 {
 	int i;
 	bool all_zero = true;
 
+	/* Any CRC value can be considered valid on amdgpu hardware. */
+	if (is_amdgpu_device(pipe_crc->fd))
+		return;
+
 	for (i = 0; i < crc->n_words; i++) {
 		igt_warn_on_f(crc->crc[i] == 0xffffffff,
 			      "Suspicious CRC: it looks like the CRC "
@@ -841,36 +893,6 @@ static void crc_sanity_checks(igt_crc_t *crc)
 	igt_warn_on_f(all_zero, "Suspicious CRC: All values are 0.\n");
 }
 
-/**
- * igt_pipe_crc_collect_crc:
- * @pipe_crc: pipe CRC object
- * @out_crc: buffer for the captured CRC values
- *
- * Read a single CRC from @pipe_crc. This function blocks until the CRC is
- * retrieved, irrespective of whether @pipe_crc has been opened with
- * igt_pipe_crc_new() or igt_pipe_crc_new_nonblock().  @out_crc must be
- * allocated by the caller.
- *
- * This function takes care of the pipe_crc book-keeping, it will start/stop
- * the collection of the CRC.
- *
- * This function also calls the interactive debug with the "crc" domain, so you
- * can make use of this feature to actually see the screen that is being CRC'd.
- *
- * For continuous CRC collection look at igt_pipe_crc_start(),
- * igt_pipe_crc_get_crcs() and igt_pipe_crc_stop().
- */
-void igt_pipe_crc_collect_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out_crc)
-{
-	igt_debug_wait_for_keypress("crc");
-
-	igt_pipe_crc_start(pipe_crc);
-	read_one_crc(pipe_crc, out_crc);
-	igt_pipe_crc_stop(pipe_crc);
-
-	crc_sanity_checks(out_crc);
-}
-
 /**
  * igt_pipe_crc_drain:
  * @pipe_crc: pipe CRC object
@@ -898,39 +920,97 @@ void igt_pipe_crc_drain(igt_pipe_crc_t *pipe_crc)
  * @pipe_crc: pipe CRC object
  * @crc: buffer pointer for the captured CRC value
  *
- * Read a single @crc from @pipe_crc. This function does not block
- * when nonblocking CRC is requested, and will return false if no CRC
- * can be captured.
- *
- * If opened in blocking mode it will always block until a new CRC is read, like
- * igt_pipe_crc_collect_crc().
+ * Read a single @crc from @pipe_crc. This function blocks even
+ * when nonblocking CRC is requested.
  *
  * Callers must start and stop the capturing themselves by calling
  * igt_pipe_crc_start() and igt_pipe_crc_stop(). For one-shot CRC collecting
  * look at igt_pipe_crc_collect_crc().
  *
  * If capturing has been going on for a while and a fresh crc is required,
- * you will need to call igt_pipe_crc_drain() first to remove stale entries.
- *
- * Returns:
- * Whether a crc is captured, only false in non-blocking mode.
+ * you should use igt_pipe_crc_get_current() instead.
  */
-bool
-igt_pipe_crc_get_single(igt_pipe_crc_t *pipe_crc, igt_crc_t *crc)
+void igt_pipe_crc_get_single(igt_pipe_crc_t *pipe_crc, igt_crc_t *crc)
 {
-	bool found = true;
+	read_one_crc(pipe_crc, crc);
 
-	if (pipe_crc->flags & O_NONBLOCK) {
-		int ret = read_crc(pipe_crc, crc);
+	crc_sanity_checks(pipe_crc, crc);
+}
 
-		found = ret > 0;
-	} else
+/**
+ * igt_pipe_crc_get_current:
+ * @drm_fd: Pointer to drm fd for vblank counter
+ * @pipe_crc: pipe CRC object
+ * @crc: buffer pointer for the captured CRC value
+ *
+ * Same as igt_pipe_crc_get_single(), but will wait until a new CRC can be captured.
+ * This is useful for retrieving the current CRC in a more race free way than
+ * igt_pipe_crc_drain() + igt_pipe_crc_get_single().
+ */
+void
+igt_pipe_crc_get_current(int drm_fd, igt_pipe_crc_t *pipe_crc, igt_crc_t *crc)
+{
+	unsigned vblank = kmstest_get_vblank(drm_fd, pipe_crc->pipe, 0);
+
+	do {
 		read_one_crc(pipe_crc, crc);
 
-	if (found)
-		crc_sanity_checks(crc);
+		/* Only works with valid frame counter */
+		if (!crc->has_valid_frame) {
+			igt_pipe_crc_drain(pipe_crc);
+			igt_pipe_crc_get_single(pipe_crc, crc);
+			return;
+		}
+	} while (igt_vblank_before_eq(crc->frame, vblank));
 
-	return found;
+	crc_sanity_checks(pipe_crc, crc);
+}
+
+/**
+ * igt_pipe_crc_collect_crc:
+ * @pipe_crc: pipe CRC object
+ * @out_crc: buffer for the captured CRC values
+ *
+ * Read a single CRC from @pipe_crc. This function blocks until the CRC is
+ * retrieved, irrespective of whether @pipe_crc has been opened with
+ * igt_pipe_crc_new() or igt_pipe_crc_new_nonblock().  @out_crc must be
+ * allocated by the caller.
+ *
+ * This function takes care of the pipe_crc book-keeping, it will start/stop
+ * the collection of the CRC.
+ *
+ * This function also calls the interactive debug with the "crc" domain, so you
+ * can make use of this feature to actually see the screen that is being CRC'd.
+ *
+ * For continuous CRC collection look at igt_pipe_crc_start(),
+ * igt_pipe_crc_get_crcs() and igt_pipe_crc_stop().
+ */
+void igt_pipe_crc_collect_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out_crc)
+{
+	igt_debug_wait_for_keypress("crc");
+
+	igt_pipe_crc_start(pipe_crc);
+	igt_pipe_crc_get_single(pipe_crc, out_crc);
+	igt_pipe_crc_stop(pipe_crc);
+}
+
+/**
+ * igt_reset_fifo_underrun_reporting:
+ * @drm_fd: drm device file descriptor
+ *
+ * Resets fifo underrun reporting, if supported by the device. Useful since fifo
+ * underrun reporting tends to be one-shot, so good to reset it before the
+ * actual functional test again in case there's been a separate issue happening
+ * while preparing the test setup.
+ */
+void igt_reset_fifo_underrun_reporting(int drm_fd)
+{
+	int fd = igt_debugfs_open(drm_fd, "i915_fifo_underrun_reset", O_WRONLY);
+	if (fd >= 0) {
+		igt_assert_eq(write(fd, "y", 1), 1);
+
+		close(fd);
+	}
 }
 
 /*
@@ -1065,7 +1145,7 @@ int igt_get_stable_obj_count(int driver)
 	return obj_count;
 }
 
-void igt_debugfs_dump(int device, const char *filename)
+void __igt_debugfs_dump(int device, const char *filename, int level)
 {
 	char *contents;
 	int dir;
@@ -1074,6 +1154,6 @@ void igt_debugfs_dump(int device, const char *filename)
 	contents = igt_sysfs_get(dir, filename);
 	close(dir);
 
-	igt_debug("%s:\n%s\n", filename, contents);
+	igt_log(IGT_LOG_DOMAIN, level, "%s:\n%s\n", filename, contents);
 	free(contents);
 }
