@@ -33,6 +33,14 @@
 #include "igt_fb.h"
 #include "ioctl_wrappers.h"
 #include "i830_reg.h"
+#include "i915/gem_mman.h"
+
+#ifndef PAGE_ALIGN
+#ifndef PAGE_SIZE
+#define PAGE_SIZE 4096
+#endif
+#define PAGE_ALIGN(x) ALIGN(x, PAGE_SIZE)
+#endif
 
 /**
  * SECTION:igt_draw
@@ -336,7 +344,7 @@ static void draw_rect_mmap_cpu(int fd, struct buf_data *buf, struct rect *rect,
 	if (tiling != I915_TILING_NONE)
 		igt_require(intel_gen(intel_get_drm_devid(fd)) >= 5);
 
-	ptr = gem_mmap__cpu(fd, buf->handle, 0, buf->size, 0);
+	ptr = gem_mmap__cpu(fd, buf->handle, 0, PAGE_ALIGN(buf->size), 0);
 
 	switch (tiling) {
 	case I915_TILING_NONE:
@@ -365,7 +373,8 @@ static void draw_rect_mmap_gtt(int fd, struct buf_data *buf, struct rect *rect,
 	gem_set_domain(fd, buf->handle, I915_GEM_DOMAIN_GTT,
 		       I915_GEM_DOMAIN_GTT);
 
-	ptr = gem_mmap__gtt(fd, buf->handle, buf->size, PROT_READ | PROT_WRITE);
+	ptr = gem_mmap__gtt(fd, buf->handle, PAGE_ALIGN(buf->size),
+			    PROT_READ | PROT_WRITE);
 
 	draw_rect_ptr_linear(ptr, buf->stride, rect, color, buf->bpp);
 
@@ -386,7 +395,7 @@ static void draw_rect_mmap_wc(int fd, struct buf_data *buf, struct rect *rect,
 	if (tiling != I915_TILING_NONE)
 		igt_require(intel_gen(intel_get_drm_devid(fd)) >= 5);
 
-	ptr = gem_mmap__wc(fd, buf->handle, 0, buf->size,
+	ptr = gem_mmap__wc(fd, buf->handle, 0, PAGE_ALIGN(buf->size),
 			   PROT_READ | PROT_WRITE);
 
 	switch (tiling) {
@@ -573,18 +582,9 @@ static void draw_rect_render(int fd, struct cmd_data *cmd_data,
 	uint32_t tiling, swizzle;
 	struct buf_data tmp;
 	int pixel_size = buf->bpp / 8;
-	unsigned adjusted_w, adjusted_dst_x;
 
 	igt_skip_on(!rendercopy);
 
-	/* Rendercopy works at 32bpp, so if you try to do copies on buffers with
-	 * smaller bpps you won't succeeed if you need to copy "half" of a 32bpp
-	 * pixel or something similar. */
-	igt_skip_on(rect->x % (32 / buf->bpp) != 0 ||
-		    rect->y % (32 / buf->bpp) != 0 ||
-		    rect->w % (32 / buf->bpp) != 0 ||
-		    rect->h % (32 / buf->bpp) != 0);
-
 	igt_require(gem_get_tiling(fd, buf->handle, &tiling, &swizzle));
 
 	/* We create a temporary buffer and copy from it using rendercopy. */
@@ -604,26 +604,18 @@ static void draw_rect_render(int fd, struct cmd_data *cmd_data,
 	src_buf.stride = tmp.stride;
 	src_buf.tiling = I915_TILING_NONE;
 	src_buf.size = tmp.size;
+	src_buf.bpp = tmp.bpp;
 	dst_buf.bo = dst;
 	dst_buf.stride = buf->stride;
 	dst_buf.tiling = tiling;
 	dst_buf.size = buf->size;
+	dst_buf.bpp = buf->bpp;
 
 	batch = intel_batchbuffer_alloc(cmd_data->bufmgr, devid);
 	igt_assert(batch);
 
-	switch (buf->bpp) {
-	case 16:
-	case 32:
-		adjusted_w = rect->w / (32 / buf->bpp);
-		adjusted_dst_x = rect->x / (32 / buf->bpp);
-		break;
-	default:
-		igt_assert(false);
-	}
-
-	rendercopy(batch, cmd_data->context, &src_buf, 0, 0, adjusted_w,
-		   rect->h, &dst_buf, adjusted_dst_x, rect->y);
+	rendercopy(batch, cmd_data->context, &src_buf, 0, 0, rect->w,
+		   rect->h, &dst_buf, rect->x, rect->y);
 
 	intel_batchbuffer_free(batch);
 	drm_intel_bo_unreference(src);
@@ -720,7 +712,7 @@ void igt_draw_rect_fb(int fd, drm_intel_bufmgr *bufmgr,
 		      enum igt_draw_method method, int rect_x, int rect_y,
 		      int rect_w, int rect_h, uint32_t color)
 {
-	igt_draw_rect(fd, bufmgr, context, fb->gem_handle, fb->size, fb->stride,
+	igt_draw_rect(fd, bufmgr, context, fb->gem_handle, fb->size, fb->strides[0],
 		      method, rect_x, rect_y, rect_w, rect_h, color,
 		      igt_drm_format_to_bpp(fb->drm_format));
 }
