@@ -547,10 +547,12 @@ int __gem_create(int fd, uint64_t size, uint32_t *handle)
 	};
 	int err = 0;
 
-	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create) == 0)
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create) == 0) {
 		*handle = create.handle;
-	else
+	} else {
 		err = -errno;
+		igt_assume(err != 0);
+	}
 
 	errno = 0;
 	return err;
@@ -586,8 +588,10 @@ uint32_t gem_create(int fd, uint64_t size)
 int __gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
 {
 	int err = 0;
-	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, execbuf))
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, execbuf)) {
 		err = -errno;
+		igt_assume(err != 0);
+	}
 	errno = 0;
 	return err;
 }
@@ -616,8 +620,10 @@ void gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
 int __gem_execbuf_wr(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
 {
 	int err = 0;
-	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2_WR, execbuf))
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2_WR, execbuf)) {
 		err = -errno;
+		igt_assume(err != 0);
+	}
 	errno = 0;
 	return err;
 }
@@ -635,213 +641,6 @@ void gem_execbuf_wr(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
 	igt_assert_eq(__gem_execbuf_wr(fd, execbuf), 0);
 }
 
-/**
- * __gem_mmap__gtt:
- * @fd: open i915 drm file descriptor
- * @handle: gem buffer object handle
- * @size: size of the gem buffer
- * @prot: memory protection bits as used by mmap()
- *
- * This functions wraps up procedure to establish a memory mapping through the
- * GTT.
- *
- * Returns: A pointer to the created memory mapping, NULL on failure.
- */
-void *__gem_mmap__gtt(int fd, uint32_t handle, uint64_t size, unsigned prot)
-{
-	struct drm_i915_gem_mmap_gtt mmap_arg;
-	void *ptr;
-
-	memset(&mmap_arg, 0, sizeof(mmap_arg));
-	mmap_arg.handle = handle;
-	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP_GTT, &mmap_arg))
-		return NULL;
-
-	ptr = mmap64(0, size, prot, MAP_SHARED, fd, mmap_arg.offset);
-	if (ptr == MAP_FAILED)
-		ptr = NULL;
-	else
-		errno = 0;
-
-	VG(VALGRIND_MAKE_MEM_DEFINED(ptr, size));
-
-	return ptr;
-}
-
-/**
- * gem_mmap__gtt:
- * @fd: open i915 drm file descriptor
- * @handle: gem buffer object handle
- * @size: size of the gem buffer
- * @prot: memory protection bits as used by mmap()
- *
- * Like __gem_mmap__gtt() except we assert on failure.
- *
- * Returns: A pointer to the created memory mapping
- */
-void *gem_mmap__gtt(int fd, uint32_t handle, uint64_t size, unsigned prot)
-{
-	void *ptr = __gem_mmap__gtt(fd, handle, size, prot);
-	igt_assert(ptr);
-	return ptr;
-}
-
-int gem_munmap(void *ptr, uint64_t size)
-{
-	int ret = munmap(ptr, size);
-
-	if (ret == 0)
-		VG(VALGRIND_MAKE_MEM_NOACCESS(ptr, size));
-
-	return ret;
-}
-
-bool gem_mmap__has_wc(int fd)
-{
-	static int has_wc = -1;
-
-	if (has_wc == -1) {
-		struct drm_i915_getparam gp;
-		int mmap_version = -1;
-		int gtt_version = -1;
-
-		has_wc = 0;
-
-		memset(&gp, 0, sizeof(gp));
-		gp.param = 40; /* MMAP_GTT_VERSION */
-		gp.value = &gtt_version;
-		ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp);
-
-		memset(&gp, 0, sizeof(gp));
-		gp.param = 30; /* MMAP_VERSION */
-		gp.value = &mmap_version;
-		ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp);
-
-		/* Do we have the new mmap_ioctl with DOMAIN_WC? */
-		if (mmap_version >= 1 && gtt_version >= 2) {
-			struct drm_i915_gem_mmap arg;
-
-			/* Does this device support wc-mmaps ? */
-			memset(&arg, 0, sizeof(arg));
-			arg.handle = gem_create(fd, 4096);
-			arg.offset = 0;
-			arg.size = 4096;
-			arg.flags = I915_MMAP_WC;
-			has_wc = igt_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP, &arg) == 0;
-			gem_close(fd, arg.handle);
-		}
-		errno = 0;
-	}
-
-	return has_wc > 0;
-}
-
-/**
- * __gem_mmap__wc:
- * @fd: open i915 drm file descriptor
- * @handle: gem buffer object handle
- * @offset: offset in the gem buffer of the mmap arena
- * @size: size of the mmap arena
- * @prot: memory protection bits as used by mmap()
- *
- * This functions wraps up procedure to establish a memory mapping through
- * direct cpu access, bypassing the gpu and cpu caches completely and also
- * bypassing the GTT system agent (i.e. there is no automatic tiling of
- * the mmapping through the fence registers).
- *
- * Returns: A pointer to the created memory mapping, NULL on failure.
- */
-void *__gem_mmap__wc(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
-{
-	struct drm_i915_gem_mmap arg;
-
-	if (!gem_mmap__has_wc(fd)) {
-		errno = ENOSYS;
-		return NULL;
-	}
-
-	memset(&arg, 0, sizeof(arg));
-	arg.handle = handle;
-	arg.offset = offset;
-	arg.size = size;
-	arg.flags = I915_MMAP_WC;
-	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP, &arg))
-		return NULL;
-
-	VG(VALGRIND_MAKE_MEM_DEFINED(from_user_pointer(arg.addr_ptr), arg.size));
-
-	errno = 0;
-	return from_user_pointer(arg.addr_ptr);
-}
-
-/**
- * gem_mmap__wc:
- * @fd: open i915 drm file descriptor
- * @handle: gem buffer object handle
- * @offset: offset in the gem buffer of the mmap arena
- * @size: size of the mmap arena
- * @prot: memory protection bits as used by mmap()
- *
- * Like __gem_mmap__wc() except we assert on failure.
- *
- * Returns: A pointer to the created memory mapping
- */
-void *gem_mmap__wc(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
-{
-	void *ptr = __gem_mmap__wc(fd, handle, offset, size, prot);
-	igt_assert(ptr);
-	return ptr;
-}
-
-/**
- * __gem_mmap__cpu:
- * @fd: open i915 drm file descriptor
- * @handle: gem buffer object handle
- * @offset: offset in the gem buffer of the mmap arena
- * @size: size of the mmap arena
- * @prot: memory protection bits as used by mmap()
- *
- * This functions wraps up procedure to establish a memory mapping through
- * direct cpu access, bypassing the gpu completely.
- *
- * Returns: A pointer to the created memory mapping, NULL on failure.
- */
-void *__gem_mmap__cpu(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
-{
-	struct drm_i915_gem_mmap mmap_arg;
-
-	memset(&mmap_arg, 0, sizeof(mmap_arg));
-	mmap_arg.handle = handle;
-	mmap_arg.offset = offset;
-	mmap_arg.size = size;
-	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP, &mmap_arg))
-		return NULL;
-
-	VG(VALGRIND_MAKE_MEM_DEFINED(from_user_pointer(mmap_arg.addr_ptr), mmap_arg.size));
-
-	errno = 0;
-	return from_user_pointer(mmap_arg.addr_ptr);
-}
-
-/**
- * gem_mmap__cpu:
- * @fd: open i915 drm file descriptor
- * @handle: gem buffer object handle
- * @offset: offset in the gem buffer of the mmap arena
- * @size: size of the mmap arena
- * @prot: memory protection bits as used by mmap()
- *
- * Like __gem_mmap__cpu() except we assert on failure.
- *
- * Returns: A pointer to the created memory mapping
- */
-void *gem_mmap__cpu(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
-{
-	void *ptr = __gem_mmap__cpu(fd, handle, offset, size, prot);
-	igt_assert(ptr);
-	return ptr;
-}
-
 /**
  * gem_madvise:
  * @fd: open i915 drm file descriptor
@@ -954,18 +753,17 @@ bool gem_bo_busy(int fd, uint32_t handle)
  * Feature test macro to check what type of gtt is being used by the kernel:
  * 0 - global gtt
  * 1 - aliasing ppgtt
- * 2 - full ppgtt, limited to 32bit address space
- * 3 - full ppgtt, 64bit address space
+ * 2 - full ppgtt
  *
  * Returns: Type of gtt being used.
  */
-int gem_gtt_type(int fd)
+static int gem_gtt_type(int fd)
 {
 	struct drm_i915_getparam gp;
 	int val = 0;
 
 	memset(&gp, 0, sizeof(gp));
-	gp.param = 18; /* HAS_ALIASING_PPGTT */
+	gp.param = I915_PARAM_HAS_ALIASING_PPGTT;
 	gp.value = &val;
 
 	if (ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp)))
@@ -1454,25 +1252,6 @@ void igt_require_gem(int fd)
 	igt_require_f(err == 0, "Unresponsive i915/GEM device\n");
 }
 
-bool gem_has_ring(int fd, unsigned ring)
-{
-	struct drm_i915_gem_execbuffer2 execbuf;
-	struct drm_i915_gem_exec_object2 exec;
-
-	/* silly ABI, the kernel thinks everyone who has BSD also has BSD2 */
-	if ((ring & ~(3<<13)) == I915_EXEC_BSD) {
-		if (ring & (3 << 13) && !gem_has_bsd2(fd))
-			return false;
-	}
-
-	memset(&exec, 0, sizeof(exec));
-	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = to_user_pointer(&exec);
-	execbuf.buffer_count = 1;
-	execbuf.flags = ring;
-	return __gem_execbuf(fd, &execbuf) == -ENOENT;
-}
-
 /**
  * gem_require_ring:
  * @fd: open i915 drm file descriptor
@@ -1647,13 +1426,7 @@ void prime_sync_end(int dma_buf_fd, bool write)
 	do_ioctl(dma_buf_fd, LOCAL_DMA_BUF_IOCTL_SYNC, &sync_end);
 }
 
-/**
- * igt_require_fb_modifiers:
- * @fd: Open DRM file descriptor.
- *
- * Requires presence of DRM_CAP_ADDFB2_MODIFIERS.
- */
-void igt_require_fb_modifiers(int fd)
+bool igt_has_fb_modifiers(int fd)
 {
 	static bool has_modifiers, cap_modifiers_tested;
 
@@ -1662,22 +1435,36 @@ void igt_require_fb_modifiers(int fd)
 		int ret;
 
 		ret = drmGetCap(fd, DRM_CAP_ADDFB2_MODIFIERS, &cap_modifiers);
-		igt_assert(ret == 0 || errno == EINVAL);
+		igt_assert(ret == 0 || errno == EINVAL || errno == EOPNOTSUPP);
 		has_modifiers = ret == 0 && cap_modifiers == 1;
 		cap_modifiers_tested = true;
 	}
 
-	igt_require(has_modifiers);
+	return has_modifiers;
 }
 
-int __kms_addfb(int fd, uint32_t handle, uint32_t width, uint32_t height,
-		uint32_t stride, uint32_t pixel_format, uint64_t modifier,
-		uint32_t *offsets, uint32_t flags, uint32_t *buf_id)
+/**
+ * igt_require_fb_modifiers:
+ * @fd: Open DRM file descriptor.
+ *
+ * Requires presence of DRM_CAP_ADDFB2_MODIFIERS.
+ */
+void igt_require_fb_modifiers(int fd)
+{
+	igt_require(igt_has_fb_modifiers(fd));
+}
+
+int __kms_addfb(int fd, uint32_t handle,
+		uint32_t width, uint32_t height,
+		uint32_t pixel_format, uint64_t modifier,
+		uint32_t strides[4], uint32_t offsets[4],
+		int num_planes, uint32_t flags, uint32_t *buf_id)
 {
 	struct drm_mode_fb_cmd2 f;
 	int ret, i;
 
-	igt_require_fb_modifiers(fd);
+	if (flags & DRM_MODE_FB_MODIFIERS)
+		igt_require_fb_modifiers(fd);
 
 	memset(&f, 0, sizeof(f));
 
@@ -1685,14 +1472,11 @@ int __kms_addfb(int fd, uint32_t handle, uint32_t width, uint32_t height,
 	f.height = height;
 	f.pixel_format = pixel_format;
 	f.flags = flags;
-	f.handles[0] = handle;
-	f.pitches[0] = stride;
-	f.modifier[0] = modifier;
 
-	for (i = 1; i < 4 && offsets && offsets[i]; i++) {
+	for (i = 0; i < num_planes; i++) {
 		f.handles[i] = handle;
-		f.pitches[i] = stride;
 		f.modifier[i] = modifier;
+		f.pitches[i] = strides[i];
 		f.offsets[i] = offsets[i];
 	}
 
