@@ -43,6 +43,8 @@
 #define DRM_CAP_CURSOR_HEIGHT 0x9
 #endif
 
+#define PAGE_SIZE	4096
+
 IGT_TEST_DESCRIPTION("Stress legacy cursor ioctl");
 
 igt_pipe_crc_t *pipe_crc;
@@ -63,7 +65,7 @@ static void stress(igt_display_t *display,
 		num_children = -num_children;
 	}
 
-	results = mmap(NULL, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+	results = mmap(NULL, PAGE_SIZE, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
 	igt_assert(results != MAP_FAILED);
 
 	memset(&arg, 0, sizeof(arg));
@@ -146,7 +148,7 @@ static void stress(igt_display_t *display,
 	}
 
 	gem_close(display->drm_fd, arg.handle);
-	munmap(results, 4096);
+	munmap(results, PAGE_SIZE);
 }
 
 static igt_output_t *set_fb_on_crtc(igt_display_t *display, enum pipe pipe, struct igt_fb *fb_info)
@@ -387,7 +389,7 @@ static void flip(igt_display_t *display,
 	uint64_t *results;
 	struct igt_fb fb_info, fb_info2, argb_fb, cursor_fb, cursor_fb2;
 
-	results = mmap(NULL, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+	results = mmap(NULL, PAGE_SIZE, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
 	igt_assert(results != MAP_FAILED);
 
 	flip_pipe = find_connected_pipe(display, !!flip_pipe);
@@ -451,7 +453,7 @@ static void flip(igt_display_t *display,
 	}
 	igt_waitchildren();
 
-	munmap(results, 4096);
+	munmap(results, PAGE_SIZE);
 
 	igt_remove_fb(display->drm_fd, &fb_info);
 	if (flip_pipe != cursor_pipe)
@@ -532,8 +534,8 @@ static void basic_flip_cursor(igt_display_t *display,
 
 		spin = NULL;
 		if (flags & BASIC_BUSY)
-			spin = igt_spin_batch_new(display->drm_fd,
-						  .dependency = fb_info.gem_handle);
+			spin = igt_spin_new(display->drm_fd,
+					    .dependency = fb_info.gem_handle);
 
 		/* Start with a synchronous query to align with the vblank */
 		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
@@ -578,7 +580,7 @@ static void basic_flip_cursor(igt_display_t *display,
 		if (spin) {
 			struct pollfd pfd = { display->drm_fd, POLLIN };
 			igt_assert(poll(&pfd, 1, 0) == 0);
-			igt_spin_batch_free(display->drm_fd, spin);
+			igt_spin_free(display->drm_fd, spin);
 		}
 
 		if (miss)
@@ -642,7 +644,7 @@ get_cursor_updates_per_vblank(igt_display_t *display, enum pipe pipe,
 	target /= 4;
 	igt_require(target > 1);
 
-	igt_debug("Using a target of %d cursor updates per quarter-vblank\n", target);
+	igt_info("Using a target of %d cursor updates per quarter-vblank\n", target);
 
 	return target;
 }
@@ -690,7 +692,7 @@ static void flip_vs_cursor(igt_display_t *display, enum flip_test mode, int nloo
 	 * variations caused by cpu speed changing are eliminated.
 	 */
 	if (target > 1) {
-		shared = mmap(NULL, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+		shared = mmap(NULL, PAGE_SIZE, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
 		igt_assert(shared != MAP_FAILED);
 
 		cpu = sched_getcpu();
@@ -754,7 +756,7 @@ static void flip_vs_cursor(igt_display_t *display, enum flip_test mode, int nloo
 	if (target > 1) {
 		shared[0] = 1;
 		igt_waitchildren();
-		munmap((void *)shared, 4096);
+		munmap((void *)shared, PAGE_SIZE);
 		sched_setaffinity(0, sizeof(oldmask), &oldmask);
 	}
 
@@ -857,7 +859,7 @@ static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool
 	enum pipe pipe = find_connected_pipe(display, false);
 	enum pipe pipe2 = find_connected_pipe(display, true);
 	igt_output_t *output, *output2;
-	bool vblank_matches, enabled = false;
+	bool enabled = false;
 	volatile unsigned long *shared;
 	unsigned flags = 0, vblank_start;
 	struct drm_event_vblank vbl;
@@ -870,7 +872,7 @@ static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool
 		igt_require(drmGetCap(display->drm_fd, DRM_CAP_CRTC_IN_VBLANK_EVENT, &val) == 0);
 	}
 
-	shared = mmap(NULL, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+	shared = mmap(NULL, PAGE_SIZE, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
 	igt_assert(shared != MAP_FAILED);
 
 	igt_fail_on(modeset && !atomic);
@@ -961,7 +963,6 @@ static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool
 		flip_nonblocking(display, pipe2, atomic, &fb2_info, (void*)(ptrdiff_t)vblank_start);
 	}
 
-	vblank_matches = false;
 	while (nloops) {
 		shared[1] = nloops & 1;
 
@@ -976,9 +977,6 @@ static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool
 		if (!modeset)
 			igt_assert_eq(vbl.sequence, vblank_start + 1);
 
-		if (vblank_start && vbl.sequence == vblank_start + 1)
-			vblank_matches = true;
-
 		/* Do not requeue on the last 2 events. */
 		if (nloops <= 2) {
 			nloops--;
@@ -1011,8 +1009,6 @@ static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool
 		}
 	}
 
-	igt_assert_f(vblank_matches, "During modeset at least 1 page flip needs to match!\n");
-
 done:
 	shared[0] = 1;
 	igt_waitchildren();
@@ -1020,6 +1016,7 @@ done:
 	igt_remove_fb(display->drm_fd, &fb_info);
 	igt_remove_fb(display->drm_fd, &fb2_info);
 	igt_remove_fb(display->drm_fd, &cursor_fb);
+	munmap((void *)shared, PAGE_SIZE);
 }
 
 static void cursor_vs_flip(igt_display_t *display, enum flip_test mode, int nloops)
@@ -1033,11 +1030,12 @@ static void cursor_vs_flip(igt_display_t *display, enum flip_test mode, int nloo
 	enum pipe pipe = find_connected_pipe(display, false);
 	igt_output_t *output;
 	uint32_t vrefresh;
+	int fail_count;
 
 	if (mode >= flip_test_atomic)
 		igt_require(display->is_atomic);
 
-	shared = mmap(NULL, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+	shared = mmap(NULL, PAGE_SIZE, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
 	igt_assert(shared != MAP_FAILED);
 
 	igt_require((output = set_fb_on_crtc(display, pipe, &fb_info)));
@@ -1053,6 +1051,8 @@ static void cursor_vs_flip(igt_display_t *display, enum flip_test mode, int nloo
 
 	target = get_cursor_updates_per_vblank(display, pipe, &arg[0]);
 
+	fail_count = 0;
+
 	for (int i = 0; i < nloops; i++) {
 		shared[0] = 0;
 		igt_fork(child, 1) {
@@ -1093,17 +1093,23 @@ static void cursor_vs_flip(igt_display_t *display, enum flip_test mode, int nloo
 
 		shared[0] = 1;
 		igt_waitchildren();
-		igt_assert_f(shared[0] > vrefresh*target / 2,
-			     "completed %lu cursor updated in a period of %u flips, "
-			     "we expect to complete approximately %lu updates, "
-			     "with the threshold set at %lu\n",
-			     shared[0], vrefresh / 2,
-			     vrefresh*target, vrefresh*target / 2);
+		if (shared[0] <= vrefresh*target / 2) {
+			fail_count++;
+			igt_critical("completed %lu cursor updated in a period of %u flips, "
+				     "we expect to complete approximately %lu updates, "
+				     "with the threshold set at %lu\n",
+				     shared[0], vrefresh / 2,
+				     vrefresh*target, vrefresh*target / 2);
+		}
 	}
 
+	igt_assert_f(fail_count == 0,
+		     "Failed to meet cursor update expectations in %d out of %d iterations\n",
+		     fail_count, nloops);
+
 	igt_remove_fb(display->drm_fd, &fb_info);
 	igt_remove_fb(display->drm_fd, &cursor_fb);
-	munmap((void *)shared, 4096);
+	munmap((void *)shared, PAGE_SIZE);
 	if (argb_fb.gem_handle)
 		igt_remove_fb(display->drm_fd, &argb_fb);
 	if (cursor_fb2.gem_handle)
@@ -1123,7 +1129,7 @@ static void two_screens_cursor_vs_flip(igt_display_t *display, int nloops, bool
 	};
 	igt_output_t *outputs[2];
 
-	shared = mmap(NULL, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+	shared = mmap(NULL, PAGE_SIZE, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
 	igt_assert(shared != MAP_FAILED);
 
 	if (atomic)
@@ -1210,7 +1216,7 @@ static void two_screens_cursor_vs_flip(igt_display_t *display, int nloops, bool
 	igt_remove_fb(display->drm_fd, &fb_info[0]);
 	igt_remove_fb(display->drm_fd, &fb_info[1]);
 	igt_remove_fb(display->drm_fd, &cursor_fb);
-	munmap((void *)shared, 4096);
+	munmap((void *)shared, PAGE_SIZE);
 }
 
 static void flip_vs_cursor_crc(igt_display_t *display, bool atomic)
@@ -1324,8 +1330,8 @@ static void flip_vs_cursor_busy_crc(igt_display_t *display, bool atomic)
 	for (int i = 1; i >= 0; i--) {
 		igt_spin_t *spin;
 
-		spin = igt_spin_batch_new(display->drm_fd,
-					  .dependency = fb_info[1].gem_handle);
+		spin = igt_spin_new(display->drm_fd,
+				    .dependency = fb_info[1].gem_handle);
 
 		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
 
@@ -1334,10 +1340,9 @@ static void flip_vs_cursor_busy_crc(igt_display_t *display, bool atomic)
 
 		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
 
-		igt_pipe_crc_drain(pipe_crc);
-		igt_pipe_crc_get_single(pipe_crc, &test_crc);
+		igt_pipe_crc_get_current(display->drm_fd, pipe_crc, &test_crc);
 
-		igt_spin_batch_free(display->drm_fd, spin);
+		igt_spin_free(display->drm_fd, spin);
 
 		igt_set_timeout(1, "Stuck page flip");
 		igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
@@ -1366,8 +1371,7 @@ igt_main
 		display.drm_fd = drm_open_driver_master(DRIVER_ANY);
 		kmstest_set_vt_graphics_mode();
 
-		igt_display_init(&display, display.drm_fd);
-		igt_require(display.n_pipes > 0);
+		igt_display_require(&display, display.drm_fd);
 	}
 
 	igt_subtest_group {
