@@ -60,7 +60,7 @@
 #define TEST_RMFB		(1 << 14)
 #define TEST_HANG		(1 << 15)
 #define TEST_NOEVENT		(1 << 16)
-#define TEST_FB_BAD_TILING	(1 << 17)
+
 #define TEST_SINGLE_BUFFER	(1 << 18)
 #define TEST_DPMS_OFF		(1 << 19)
 #define TEST_NO_2X_OUTPUT	(1 << 20)
@@ -68,16 +68,18 @@
 #define TEST_ENOENT		(1 << 22)
 #define TEST_FENCE_STRESS	(1 << 23)
 #define TEST_VBLANK_RACE	(1 << 24)
-#define TEST_RPM		(1 << 25)
 #define TEST_SUSPEND		(1 << 26)
-#define TEST_TS_CONT		(1 << 27)
 #define TEST_BO_TOOBIG		(1 << 28)
 
+#define TEST_NO_VBLANK		(1 << 29)
 #define TEST_BASIC		(1 << 30)
 
 #define EVENT_FLIP		(1 << 0)
 #define EVENT_VBLANK		(1 << 1)
 
+#define RUN_TEST		1
+#define RUN_PAIR		2
+
 #ifndef DRM_CAP_TIMESTAMP_MONOTONIC
 #define DRM_CAP_TIMESTAMP_MONOTONIC 6
 #endif
@@ -125,6 +127,18 @@ struct event_state {
 	int seq_step;
 };
 
+static bool vblank_dependence(int flags)
+{
+	int vblank_flags = TEST_VBLANK | TEST_VBLANK_BLOCK |
+			   TEST_VBLANK_ABSOLUTE | TEST_VBLANK_EXPIRED_SEQ |
+			   TEST_CHECK_TS | TEST_VBLANK_RACE | TEST_EBUSY;
+
+	if (flags & vblank_flags)
+		return true;
+
+	return false;
+}
+
 static float timeval_float(const struct timeval *tv)
 {
 	return tv->tv_sec + tv->tv_usec / 1000000.0f;
@@ -493,26 +507,11 @@ static void check_state(const struct test_output *o, const struct event_state *e
 	/* check only valid if no modeset happens in between, that increments by
 	 * (1 << 23) on each step. This bounding matches the one in
 	 * DRM_IOCTL_WAIT_VBLANK. */
-	if (!(o->flags & (TEST_DPMS | TEST_MODESET)))
+	if (!(o->flags & (TEST_DPMS | TEST_MODESET | TEST_NO_VBLANK)))
 		igt_assert_f(es->current_seq - (es->last_seq + o->seq_step) <= 1UL << 23,
 			     "unexpected %s seq %u, should be >= %u\n",
 			     es->name, es->current_seq, es->last_seq + o->seq_step);
 
-	/* Check that the vblank frame didn't wrap unexpectedly. */
-	if (o->flags & TEST_TS_CONT) {
-		/* Ignore seq_step here since vblank waits time out immediately
-		 * when we kill the crtc. */
-		igt_assert_f(es->current_seq - es->last_seq >= 0,
-			     "unexpected %s seq %u, should be >= %u\n",
-			     es->name, es->current_seq, es->last_seq);
-		igt_assert_f(es->current_seq - es->last_seq <= 150,
-			     "unexpected %s seq %u, should be < %u\n",
-			     es->name, es->current_seq, es->last_seq + 150);
-
-		igt_debug("testing ts continuity: Current frame %u, old frame %u\n",
-			  es->current_seq, es->last_seq);
-	}
-
 	if (o->flags & TEST_CHECK_TS) {
 		double elapsed, expected;
 
@@ -592,7 +591,7 @@ static void recreate_fb(struct test_output *o)
 	igt_assert(r);
 
 	do_or_die(drmModeAddFB(drm_fd, o->fb_width, o->fb_height, o->depth,
-			       o->bpp, fb_info->stride,
+			       o->bpp, fb_info->strides[0],
 			       r->handle, &new_fb_id));
 
 	gem_close(drm_fd, r->handle);
@@ -603,20 +602,6 @@ static void recreate_fb(struct test_output *o)
 	o->fb_info[o->current_fb_id].fb_id = new_fb_id;
 }
 
-static void set_y_tiling(struct test_output *o, int fb_idx)
-{
-	drmModeFBPtr r;
-	struct igt_fb *fb_info = &o->fb_info[fb_idx];
-
-	/* Call rmfb/getfb/addfb to ensure those don't introduce stalls */
-	r = drmModeGetFB(drm_fd, fb_info->fb_id);
-	igt_assert(r);
-	/* Newer kernels don't allow such shenagians any more, so skip the test. */
-	igt_require(__gem_set_tiling(drm_fd, r->handle, I915_TILING_Y, fb_info->stride) == 0);
-	gem_close(drm_fd, r->handle);
-	drmFree(r);
-}
-
 static igt_hang_t hang_gpu(int fd)
 {
 	return igt_hang_ring(fd, I915_EXEC_DEFAULT);
@@ -637,22 +622,24 @@ static bool is_wedged(int fd)
 
 static int set_mode(struct test_output *o, uint32_t fb, int x, int y)
 {
-	int n;
+	int n, ret;
 
 	for (n = o->count - 1; n >= 0; n--) {
+		uint32_t buffer_id = fb, x_crtc = x, y_crtc = y;
+		uint32_t *conn = &o->_connector[n];
+		int count = 1;
+		drmModeModeInfoPtr mode = &o->kmode[n];
+
 		if (fb == 0) {
-			int ret = drmModeSetCrtc(drm_fd, o->_crtc[n],
-						 0, 0, 0,
-						 0, 0, 0);
-			if (ret)
-				return ret;
-		} else {
-			int ret = drmModeSetCrtc(drm_fd, o->_crtc[n],
-						 fb, x, y,
-						 &o->_connector[n], 1, &o->kmode[n]);
-			if (ret)
-				return ret;
+			buffer_id = x_crtc = y_crtc = count = 0;
+			conn = NULL; mode = NULL;
 		}
+
+		ret = drmModeSetCrtc(drm_fd, o->_crtc[n],
+				     buffer_id, x_crtc, y_crtc,
+				     conn, count, mode);
+		if (ret)
+			return ret;
 	}
 
 	return 0;
@@ -695,9 +682,6 @@ static unsigned int run_test_step(struct test_output *o)
 		recreate_fb(o);
 	new_fb_id = o->fb_ids[o->current_fb_id];
 
-	if (o->flags & TEST_FB_BAD_TILING)
-		new_fb_id = o->fb_ids[2];
-
 	if ((o->flags & TEST_VBLANK_EXPIRED_SEQ) &&
 	    !(o->pending_events & EVENT_VBLANK) && o->flip_state.count > 0) {
 		struct vblank_reply reply;
@@ -710,25 +694,21 @@ static unsigned int run_test_step(struct test_output *o)
 					    TEST_VBLANK_BLOCK, o->pipe, exp_seq,
 					    0, &reply));
 		igt_assert(gettime_us() - start < 500);
-		igt_assert(reply.sequence == exp_seq);
+		igt_assert_eq(reply.sequence, exp_seq);
 		igt_assert(timercmp(&reply.ts, &o->flip_state.last_ts, ==));
 	}
 
 	if (o->flags & TEST_ENOENT) {
 		/* hope that fb 0xfffffff0 does not exist */
-		igt_assert(do_page_flip(o, 0xfffffff0, false) == -ENOENT);
-		igt_assert(set_mode(o, 0xfffffff0, 0, 0) == -ENOENT);
+		igt_assert_eq(do_page_flip(o, 0xfffffff0, false), -ENOENT);
+		igt_assert_eq(set_mode(o, 0xfffffff0, 0, 0), -ENOENT);
 	}
 
 	if (do_flip && (o->flags & TEST_EINVAL) && o->flip_state.count > 0)
-		igt_assert(do_page_flip(o, new_fb_id, true) == expected_einval);
-
-	if (o->flags & TEST_FB_BAD_TILING)
-		new_fb_id = o->fb_ids[o->current_fb_id];
+		igt_assert_eq(do_page_flip(o, new_fb_id, false), expected_einval);
 
 	if (do_vblank && (o->flags & TEST_EINVAL) && o->vblank_state.count > 0)
-		igt_assert(do_wait_for_vblank(o, o->pipe, target_seq, &vbl_reply)
-		       == -EINVAL);
+		igt_assert_eq(do_wait_for_vblank(o, o->pipe, target_seq, &vbl_reply), -EINVAL);
 
 	if (o->flags & TEST_VBLANK_RACE) {
 		spawn_vblank_wait_thread(o);
@@ -797,7 +777,7 @@ static unsigned int run_test_step(struct test_output *o)
 	}
 
 	if (do_flip && (o->flags & TEST_EBUSY))
-		igt_assert(do_page_flip(o, new_fb_id, true) == -EBUSY);
+		igt_assert_eq(do_page_flip(o, new_fb_id, false), -EBUSY);
 
 	if (do_flip && (o->flags & TEST_RMFB))
 		recreate_fb(o);
@@ -829,9 +809,6 @@ static unsigned int run_test_step(struct test_output *o)
 			     "failed to disable output: %s\n",
 			     strerror(errno));
 
-	if (o->flags & TEST_RPM)
-		igt_assert(igt_wait_for_pm_status(IGT_RUNTIME_PM_STATUS_SUSPENDED));
-
 	if (o->flags & TEST_SUSPEND)
 		igt_system_suspend_autoresume(SUSPEND_STATE_MEM,
 					      SUSPEND_TEST_NONE);
@@ -840,8 +817,8 @@ static unsigned int run_test_step(struct test_output *o)
 		igt_assert(do_wait_for_vblank(o, o->pipe, target_seq, &vbl_reply)
 			   == -EINVAL);
 
-	if (do_flip && (o->flags & TEST_EINVAL) && !(o->flags & TEST_FB_BAD_TILING))
-		igt_assert(do_page_flip(o, new_fb_id, true) == expected_einval);
+	if (do_flip && (o->flags & TEST_EINVAL))
+		igt_assert(do_page_flip(o, new_fb_id, false) == expected_einval);
 
 	unhang_gpu(drm_fd, hang);
 
@@ -982,8 +959,7 @@ static void paint_flip_mode(struct igt_fb *fb, bool odd_frame)
 	igt_put_cairo_ctx(drm_fd, fb, cr);
 }
 
-static int
-fb_is_bound(struct test_output *o, int fb)
+static bool fb_is_bound(struct test_output *o, int fb)
 {
 	int n;
 
@@ -993,7 +969,7 @@ fb_is_bound(struct test_output *o, int fb)
 		};
 
 		if (drmIoctl(drm_fd, DRM_IOCTL_MODE_GETCRTC, &mode))
-			return 0;
+			return false;
 
 		if (!mode.mode_valid || mode.fb_id != fb)
 			return false;
@@ -1125,7 +1101,7 @@ static void calibrate_ts(struct test_output *o, int crtc_idx)
 
 	memset(&wait, 0, sizeof(wait));
 	wait.request.type = kmstest_get_vbl_flag(crtc_idx);
-	wait.request.type |= DRM_VBLANK_ABSOLUTE | DRM_VBLANK_NEXTONMISS;
+	wait.request.type |= DRM_VBLANK_RELATIVE | DRM_VBLANK_NEXTONMISS;
 	do_or_die(drmWaitVBlank(drm_fd, &wait));
 
 	last_seq = wait.reply.sequence;
@@ -1138,8 +1114,17 @@ static void calibrate_ts(struct test_output *o, int crtc_idx)
 	wait.request.type |= DRM_VBLANK_ABSOLUTE | DRM_VBLANK_EVENT;
 	wait.request.sequence = last_seq;
 	for (n = 0; n < CALIBRATE_TS_STEPS; n++) {
+		drmVBlank check = {};
+
 		++wait.request.sequence;
 		do_or_die(drmWaitVBlank(drm_fd, &wait));
+
+		/* Double check that haven't already missed the vblank */
+		check.request.type = kmstest_get_vbl_flag(crtc_idx);
+		check.request.type |= DRM_VBLANK_RELATIVE;
+		do_or_die(drmWaitVBlank(drm_fd, &check));
+
+		igt_assert(!igt_vblank_after(check.reply.sequence, wait.request.sequence));
 	}
 
 	igt_stats_init_with_size(&stats, CALIBRATE_TS_STEPS);
@@ -1204,9 +1189,10 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	unsigned bo_size = 0;
 	uint64_t tiling;
 	int i;
+	bool vblank = true;
 
 	switch (crtc_count) {
-	case 1:
+	case RUN_TEST:
 		connector_find_preferred_mode(o->_connector[0], crtc_idxs[0], o);
 		if (!o->mode_valid)
 			return;
@@ -1217,7 +1203,7 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 			 kmstest_connector_type_str(o->kconnector[0]->connector_type),
 			 o->kconnector[0]->connector_type_id);
 		break;
-	case 2:
+	case RUN_PAIR:
 		connector_find_compatible_mode(crtc_idxs[0], crtc_idxs[1], o);
 		if (!o->mode_valid)
 			return;
@@ -1265,22 +1251,12 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	igt_assert(o->fb_ids[0]);
 	igt_assert(o->fb_ids[1]);
 
-	if (o->flags & TEST_FB_BAD_TILING) {
-		o->fb_ids[2] = igt_create_fb(drm_fd, o->fb_width, o->fb_height,
-				igt_bpp_depth_to_drm_format(o->bpp, o->depth),
-				LOCAL_I915_FORMAT_MOD_X_TILED, &o->fb_info[2]);
-		igt_require(o->fb_ids[2]);
-	}
-
 	paint_flip_mode(&o->fb_info[0], false);
 	if (!(o->flags & TEST_BO_TOOBIG))
 		paint_flip_mode(&o->fb_info[1], true);
 	if (o->fb_ids[2])
 		paint_flip_mode(&o->fb_info[2], true);
 
-	if (o->flags & TEST_FB_BAD_TILING)
-		set_y_tiling(o, 2);
-
 	for (i = 0; i < o->count; i++)
 		kmstest_dump_mode(&o->kmode[i]);
 
@@ -1297,6 +1273,14 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	}
 	igt_assert(fb_is_bound(o, o->fb_ids[0]));
 
+	vblank = kms_has_vblank(drm_fd);
+	if (!vblank) {
+		if (vblank_dependence(o->flags))
+			igt_require_f(vblank, "There is no VBlank\n");
+		else
+			o->flags |= TEST_NO_VBLANK;
+	}
+
 	/* quiescent the hw a bit so ensure we don't miss a single frame */
 	if (o->flags & TEST_CHECK_TS)
 		calibrate_ts(o, crtc_idxs[0]);
@@ -1351,11 +1335,8 @@ static int run_test(int duration, int flags)
 
 	igt_require((flags & TEST_HANG) == 0 || !is_wedged(drm_fd));
 
-	if (flags & TEST_RPM)
-		igt_require(igt_setup_runtime_pm());
-
 	resources = drmModeGetResources(drm_fd);
-	igt_assert(resources);
+	igt_require(resources);
 
 	/* Count output configurations to scale test runtime. */
 	for (i = 0; i < resources->count_connectors; i++) {
@@ -1396,7 +1377,7 @@ static int run_test(int duration, int flags)
 			o.depth = 24;
 
 			crtc_idx = n;
-			run_test_on_crtc_set(&o, &crtc_idx, 1, duration);
+			run_test_on_crtc_set(&o, &crtc_idx, RUN_TEST, duration);
 		}
 	}
 
@@ -1412,7 +1393,7 @@ static int run_pair(int duration, int flags)
 	igt_require((flags & TEST_HANG) == 0 || !is_wedged(drm_fd));
 
 	resources = drmModeGetResources(drm_fd);
-	igt_assert(resources);
+	igt_require(resources);
 
 	/* Find a pair of connected displays */
 	for (i = 0; i < resources->count_connectors; i++) {
@@ -1465,7 +1446,8 @@ static int run_pair(int duration, int flags)
 					crtc_idxs[0] = n;
 					crtc_idxs[1] = m;
 
-					run_test_on_crtc_set(&o, crtc_idxs, 2,
+					run_test_on_crtc_set(&o, crtc_idxs,
+							     RUN_PAIR,
 							     duration);
 				}
 			}
@@ -1521,7 +1503,7 @@ static void test_nonblocking_read(int in)
 	close(fd);
 }
 
-int main(int argc, char **argv)
+igt_main
 {
 	struct {
 		int duration;
@@ -1536,7 +1518,7 @@ int main(int argc, char **argv)
 		{ 30,  TEST_VBLANK | TEST_VBLANK_BLOCK | TEST_VBLANK_ABSOLUTE,
 					"blocking-absolute-wf_vblank" },
 		{ 10, TEST_FLIP | TEST_BASIC, "plain-flip" },
-		{ 30, TEST_FLIP | TEST_EBUSY , "busy-flip" },
+		{ 1, TEST_FLIP | TEST_EBUSY, "busy-flip" },
 		{ 30, TEST_FLIP | TEST_FENCE_STRESS , "flip-vs-fences" },
 		{ 30, TEST_FLIP | TEST_CHECK_TS, "plain-flip-ts-check" },
 		{ 30, TEST_FLIP | TEST_CHECK_TS | TEST_FB_RECREATE,
@@ -1556,7 +1538,6 @@ int main(int argc, char **argv)
 			TEST_CHECK_TS, "flip-vs-blocking-wf-vblank" },
 		{ 30, TEST_FLIP | TEST_MODESET | TEST_HANG | TEST_NOEVENT, "flip-vs-modeset-vs-hang" },
 		{ 30, TEST_FLIP | TEST_PAN | TEST_HANG, "flip-vs-panning-vs-hang" },
-		{ 1, TEST_FLIP | TEST_EINVAL | TEST_FB_BAD_TILING, "flip-vs-bad-tiling" },
 
 		{ 1, TEST_DPMS_OFF | TEST_MODESET | TEST_FLIP,
 					"flip-vs-dpms-off-vs-modeset" },
@@ -1567,24 +1548,25 @@ int main(int argc, char **argv)
 		{ 10, TEST_DPMS_OFF | TEST_DPMS | TEST_VBLANK_RACE, "dpms-vs-vblank-race" },
 		{ 10, TEST_MODESET | TEST_VBLANK_RACE, "modeset-vs-vblank-race" },
 		{ 0, TEST_BO_TOOBIG | TEST_NO_2X_OUTPUT, "bo-too-big" },
+		{ 10, TEST_FLIP | TEST_SUSPEND, "flip-vs-suspend" },
 	};
 	int i;
 
-	igt_subtest_init(argc, argv);
-
 	igt_fixture {
 		drm_fd = drm_open_driver_master(DRIVER_ANY);
 
-		igt_enable_connectors();
+		igt_enable_connectors(drm_fd);
 
 		kmstest_set_vt_graphics_mode();
 		igt_install_exit_handler(kms_flip_exit_handler);
 		get_timestamp_format();
 
-		bufmgr = drm_intel_bufmgr_gem_init(drm_fd, 4096);
-		if (bufmgr) {
-			devid = intel_get_drm_devid(drm_fd);
-			batch = intel_batchbuffer_alloc(bufmgr, devid);
+		if (is_i915_device(drm_fd)) {
+			bufmgr = drm_intel_bufmgr_gem_init(drm_fd, 4096);
+			if (bufmgr) {
+				devid = intel_get_drm_devid(drm_fd);
+				batch = intel_batchbuffer_alloc(bufmgr, devid);
+			}
 		}
 	}
 
@@ -1600,10 +1582,6 @@ int main(int argc, char **argv)
 		if (tests[i].flags & TEST_NO_2X_OUTPUT)
 			continue;
 
-		/* code doesn't disable all crtcs, so skip rpm tests */
-		if (tests[i].flags & TEST_RPM)
-			continue;
-
 		igt_subtest_f( "2x-%s", tests[i].name)
 			run_pair(tests[i].duration, tests[i].flags);
 	}
@@ -1616,25 +1594,21 @@ int main(int argc, char **argv)
 		    !(tests[i].flags & TEST_VBLANK_ABSOLUTE))
 			continue;
 
+		/*
+		 * -EBUSY needs to complete in a single vblank, skip them for
+		 * interruptible tests
+		 */
+		if (tests[i].flags & TEST_EBUSY)
+			continue;
+
 		igt_subtest_f( "%s-interruptible", tests[i].name)
 			run_test(tests[i].duration, tests[i].flags);
 
 		if (tests[i].flags & TEST_NO_2X_OUTPUT)
 			continue;
 
-		/* code doesn't disable all crtcs, so skip rpm tests */
-		if (tests[i].flags & TEST_RPM)
-			continue;
-
 		igt_subtest_f( "2x-%s-interruptible", tests[i].name)
 			run_pair(tests[i].duration, tests[i].flags);
 	}
 	igt_stop_signal_helper();
-
-	/*
-	 * Let drm_fd leak, since it's needed by the dpms restore
-	 * exit_handler and igt_exit() won't return.
-	 */
-
-	igt_exit();
 }
