@@ -246,7 +246,7 @@ struct {
 	.show_hidden= false,
 	.step = 0,
 	.only_pipes = PIPE_COUNT,
-	.shared_fb_x_offset = 500,
+	.shared_fb_x_offset = 248,
 	.shared_fb_y_offset = 500,
 	.tiling = LOCAL_I915_FORMAT_MOD_X_TILED,
 };
@@ -295,28 +295,10 @@ struct {
 	.stop = true,
 };
 
-drmModeModeInfo std_1024_mode = {
-	.clock = 65000,
-	.hdisplay = 1024,
-	.hsync_start = 1048,
-	.hsync_end = 1184,
-	.htotal = 1344,
-	.hskew = 0,
-	.vdisplay = 768,
-	.vsync_start = 771,
-	.vsync_end = 777,
-	.vtotal = 806,
-	.vscan = 0,
-	.vrefresh = 60,
-	.flags = 0xA,
-	.type = 0x40,
-	.name = "Custom 1024x768",
-};
-
-static drmModeModeInfo *get_connector_smallest_mode(igt_output_t *output)
+static const drmModeModeInfo *get_connector_smallest_mode(igt_output_t *output)
 {
 	drmModeConnector *c = output->config.connector;
-	drmModeModeInfo *smallest = NULL;
+	const drmModeModeInfo *smallest = NULL;
 	int i;
 
 	for (i = 0; i < c->count_modes; i++) {
@@ -331,14 +313,14 @@ static drmModeModeInfo *get_connector_smallest_mode(igt_output_t *output)
 	}
 
 	if (c->connector_type == DRM_MODE_CONNECTOR_eDP)
-		smallest = &std_1024_mode;
+		smallest = igt_std_1024_mode_get();
 
 	return smallest;
 }
 
-static drmModeModeInfo *connector_get_mode(igt_output_t *output)
+static const drmModeModeInfo *connector_get_mode(igt_output_t *output)
 {
-	drmModeModeInfo *mode = NULL;
+	const drmModeModeInfo *mode = NULL;
 
 	if (opt.small_modes)
 		mode = get_connector_smallest_mode(output);
@@ -349,7 +331,7 @@ static drmModeModeInfo *connector_get_mode(igt_output_t *output)
 	  * bugged. */
 	if (IS_HASWELL(intel_get_drm_devid(drm.fd)) &&
 	    output->config.connector->connector_type == DRM_MODE_CONNECTOR_eDP)
-		mode = &std_1024_mode;
+		mode = igt_std_1024_mode_get();
 
 	return mode;
 }
@@ -357,7 +339,7 @@ static drmModeModeInfo *connector_get_mode(igt_output_t *output)
 static void init_mode_params(struct modeset_params *params,
 			     igt_output_t *output, enum pipe pipe)
 {
-	drmModeModeInfo *mode;
+	const drmModeModeInfo *mode;
 
 	igt_output_override_mode(output, NULL);
 	mode = connector_get_mode(output);
@@ -465,7 +447,8 @@ static void create_fb(enum pixel_format pformat, int width, int height,
 		      uint64_t tiling, int plane, struct igt_fb *fb)
 {
 	uint32_t format;
-	unsigned int size, stride;
+	uint64_t size;
+	unsigned int stride;
 	uint64_t tiling_for_size;
 
 	switch (pformat) {
@@ -720,12 +703,9 @@ static void set_mode_for_params(struct modeset_params *params)
 
 static void __debugfs_read(const char *param, char *buf, int len)
 {
-	len = igt_sysfs_read(drm.debugfs, param, buf, len - 1);
-	if (len < 0) {
+	len = igt_debugfs_simple_read(drm.debugfs, param, buf, len);
+	if (len < 0)
 		igt_assert_eq(len, -ENODEV);
-		len = 0;
-	}
-	buf[len] = '\0';
 }
 
 static int __debugfs_write(const char *param, char *buf, int len)
@@ -941,8 +921,6 @@ static bool drrs_wait_until_rr_switch_to_low(void)
 
 #define fbc_enable() igt_set_module_param_int("enable_fbc", 1)
 #define fbc_disable() igt_set_module_param_int("enable_fbc", 0)
-#define psr_enable() igt_set_module_param_int("enable_psr", 1)
-#define psr_disable() igt_set_module_param_int("enable_psr", 0)
 #define drrs_enable()	drrs_set(1)
 #define drrs_disable()	drrs_set(0)
 
@@ -1131,14 +1109,14 @@ static void unset_all_crtcs(void)
 	igt_display_commit(&drm.display);
 }
 
-static void disable_features(const struct test_mode *t)
+static bool disable_features(const struct test_mode *t)
 {
 	if (t->feature == FEATURE_DEFAULT)
-		return;
+		return false;
 
 	fbc_disable();
-	psr_disable();
 	drrs_disable();
+	return psr.can_test ? psr_disable(drm.debugfs) : false;
 }
 
 static void *busy_thread_func(void *data)
@@ -1161,7 +1139,7 @@ static void start_busy_thread(struct igt_fb *fb)
 	busy_thread.stop = false;
 	busy_thread.handle = fb->gem_handle;
 	busy_thread.size = fb->size;
-	busy_thread.stride = fb->stride;
+	busy_thread.stride = fb->strides[0];
 	busy_thread.width = fb->width;
 	busy_thread.height = fb->height;
 	busy_thread.color = pick_color(fb, COLOR_PRIM_BG);
@@ -1222,7 +1200,7 @@ static void init_blue_crc(enum pixel_format format)
 
 	print_crc("Blue CRC:  ", &blue_crcs[format].crc);
 
-	unset_all_crtcs();
+	igt_display_reset(&drm.display);
 
 	igt_remove_fb(drm.fd, &blue);
 
@@ -1274,7 +1252,7 @@ static void init_crcs(enum pixel_format format,
 		print_crc("", &pattern->crcs[format][r]);
 	}
 
-	unset_all_crtcs();
+	igt_display_reset(&drm.display);
 
 	for (r = 0; r < pattern->n_rects; r++)
 		igt_remove_fb(drm.fd, &tmp_fbs[r]);
@@ -1288,7 +1266,7 @@ static void setup_drm(void)
 	drm.debugfs = igt_debugfs_dir(drm.fd);
 
 	kmstest_set_vt_graphics_mode();
-	igt_display_init(&drm.display, drm.fd);
+	igt_display_require(&drm.display, drm.fd);
 
 	drm.bufmgr = drm_intel_bufmgr_gem_init(drm.fd, 4096);
 	igt_assert(drm.bufmgr);
@@ -1421,17 +1399,6 @@ static void teardown_fbc(void)
 {
 }
 
-static bool psr_sink_has_support(void)
-{
-	char buf[256];
-
-	debugfs_read("i915_edp_psr_status", buf);
-	if (*buf == '\0') /* !HAS_PSR -> -ENODEV*/
-		return false;
-
-	return strstr(buf, "Sink_Support: yes\n");
-}
-
 static void setup_psr(void)
 {
 	if (prim_mode_params.output->config.connector->connector_type !=
@@ -1440,7 +1407,7 @@ static void setup_psr(void)
 		return;
 	}
 
-	if (!psr_sink_has_support()) {
+	if (!psr_sink_support(drm.debugfs, PSR_MODE_1)) {
 		igt_info("Can't test PSR: not supported by sink.\n");
 		return;
 	}
@@ -1542,6 +1509,7 @@ static void do_flush(const struct test_mode *t)
 
 #define DONT_ASSERT_CRC			(1 << 0)
 #define DONT_ASSERT_FEATURE_STATUS	(1 << 1)
+#define DONT_ASSERT_FBC_STATUS		(1 << 12)
 
 #define FBC_ASSERT_FLAGS		(0xF << 2)
 #define ASSERT_FBC_ENABLED		(1 << 2)
@@ -1555,9 +1523,11 @@ static void do_flush(const struct test_mode *t)
 
 #define DRRS_ASSERT_FLAGS		(7 << 8)
 #define ASSERT_DRRS_HIGH		(1 << 8)
-#define ASSERT_DRRS_LOW		(1 << 9)
+#define ASSERT_DRRS_LOW			(1 << 9)
 #define ASSERT_DRRS_INACTIVE		(1 << 10)
 
+#define ASSERT_NO_IDLE_GPU		(1 << 11)
+
 static int adjust_assertion_flags(const struct test_mode *t, int flags)
 {
 	if (!(flags & DONT_ASSERT_FEATURE_STATUS)) {
@@ -1570,7 +1540,7 @@ static int adjust_assertion_flags(const struct test_mode *t, int flags)
 			flags |= ASSERT_DRRS_HIGH;
 	}
 
-	if ((t->feature & FEATURE_FBC) == 0)
+	if ((t->feature & FEATURE_FBC) == 0 || (flags & DONT_ASSERT_FBC_STATUS))
 		flags &= ~FBC_ASSERT_FLAGS;
 	if ((t->feature & FEATURE_PSR) == 0)
 		flags &= ~PSR_ASSERT_FLAGS;
@@ -1635,9 +1605,11 @@ static void do_status_assertions(int flags)
 	}
 
 	if (flags & ASSERT_PSR_ENABLED)
-		igt_assert_f(psr_wait_entry(drm.fd), "PSR still disabled\n");
+		igt_assert_f(psr_wait_entry(drm.debugfs, PSR_MODE_1),
+			     "PSR still disabled\n");
 	else if (flags & ASSERT_PSR_DISABLED)
-		igt_assert_f(psr_active(drm.fd, false), "PSR still enabled\n");
+		igt_assert_f(psr_wait_update(drm.debugfs, PSR_MODE_1),
+			     "PSR still enabled\n");
 }
 
 static void __do_assertions(const struct test_mode *t, int flags,
@@ -1645,6 +1617,10 @@ static void __do_assertions(const struct test_mode *t, int flags,
 {
 	flags = adjust_assertion_flags(t, flags);
 
+	/* Make sure any submitted rendering is now idle. */
+	if (!(flags & ASSERT_NO_IDLE_GPU))
+		gem_quiescent_gpu(drm.fd);
+
 	igt_debug("checking asserts in line %i\n", line);
 
 	wait_user(2, "Paused before assertions.");
@@ -1689,10 +1665,18 @@ static void enable_prim_screen_and_wait(const struct test_mode *t)
 	do_assertions(ASSERT_NO_ACTION_CHANGE);
 }
 
-static void enable_scnd_screen_and_wait(const struct test_mode *t)
+static void enable_both_screens_and_wait(const struct test_mode *t)
 {
+	fill_fb_region(&prim_mode_params.primary, COLOR_PRIM_BG);
 	fill_fb_region(&scnd_mode_params.primary, COLOR_SCND_BG);
-	set_mode_for_params(&scnd_mode_params);
+
+	__set_mode_for_params(&prim_mode_params);
+	__set_mode_for_params(&scnd_mode_params);
+
+	igt_display_commit2(&drm.display, drm.display.is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
+
+	wanted_crc = &blue_crcs[t->format].crc;
+	fbc_update_last_action();
 
 	do_assertions(ASSERT_NO_ACTION_CHANGE);
 }
@@ -1711,17 +1695,21 @@ static void set_region_for_test(const struct test_mode *t,
 	do_assertions(ASSERT_NO_ACTION_CHANGE);
 }
 
-static void enable_features_for_test(const struct test_mode *t)
+static bool enable_features_for_test(const struct test_mode *t)
 {
+	bool ret = false;
+
 	if (t->feature == FEATURE_DEFAULT)
-		return;
+		return false;
 
 	if (t->feature & FEATURE_FBC)
 		fbc_enable();
 	if (t->feature & FEATURE_PSR)
-		psr_enable();
+		ret = psr_enable(drm.debugfs, PSR_MODE_1);
 	if (t->feature & FEATURE_DRRS)
 		drrs_enable();
+
+	return ret;
 }
 
 static void check_test_requirements(const struct test_mode *t)
@@ -1803,28 +1791,38 @@ static void set_crtc_fbs(const struct test_mode *t)
 static void prepare_subtest_data(const struct test_mode *t,
 				 struct draw_pattern_info *pattern)
 {
+	bool need_modeset;
+
 	check_test_requirements(t);
 
 	stop_busy_thread();
 
-	disable_features(t);
+	need_modeset = disable_features(t);
 	set_crtc_fbs(t);
 
 	if (t->screen == SCREEN_OFFSCREEN)
 		fill_fb_region(&offscreen_fb, COLOR_OFFSCREEN_BG);
 
-	unset_all_crtcs();
+	igt_display_reset(&drm.display);
+	if (need_modeset)
+		igt_display_commit(&drm.display);
 
 	init_blue_crc(t->format);
 	if (pattern)
 		init_crcs(t->format, pattern);
 
-	enable_features_for_test(t);
+	need_modeset = enable_features_for_test(t);
+	if (need_modeset)
+		igt_display_commit(&drm.display);
 }
 
 static void prepare_subtest_screens(const struct test_mode *t)
 {
-	enable_prim_screen_and_wait(t);
+	if (t->pipes == PIPE_DUAL)
+		enable_both_screens_and_wait(t);
+	else
+		enable_prim_screen_and_wait(t);
+
 	if (t->screen == SCREEN_PRIM) {
 		if (t->plane == PLANE_CUR)
 			set_region_for_test(t, &prim_mode_params.cursor);
@@ -1832,11 +1830,7 @@ static void prepare_subtest_screens(const struct test_mode *t)
 			set_region_for_test(t, &prim_mode_params.sprite);
 	}
 
-	if (t->pipes == PIPE_SINGLE)
-		return;
-
-	enable_scnd_screen_and_wait(t);
-	if (t->screen == SCREEN_SCND) {
+	if (t->pipes == PIPE_DUAL && t->screen == SCREEN_SCND) {
 		if (t->plane == PLANE_CUR)
 			set_region_for_test(t, &scnd_mode_params.cursor);
 		if (t->plane == PLANE_SPR)
@@ -1874,16 +1868,18 @@ static void rte_subtest(const struct test_mode *t)
 	do_assertions(ASSERT_FBC_DISABLED | ASSERT_PSR_DISABLED |
 		      DONT_ASSERT_CRC | ASSERT_DRRS_INACTIVE);
 
-	enable_prim_screen_and_wait(t);
+	if (t->pipes == PIPE_SINGLE)
+		enable_prim_screen_and_wait(t);
+	else
+		enable_both_screens_and_wait(t);
+
 	set_region_for_test(t, &prim_mode_params.cursor);
 	set_region_for_test(t, &prim_mode_params.sprite);
 
-	if (t->pipes == PIPE_SINGLE)
-		return;
-
-	enable_scnd_screen_and_wait(t);
-	set_region_for_test(t, &scnd_mode_params.cursor);
-	set_region_for_test(t, &scnd_mode_params.sprite);
+	if (t->pipes == PIPE_DUAL) {
+		set_region_for_test(t, &scnd_mode_params.cursor);
+		set_region_for_test(t, &scnd_mode_params.sprite);
+	}
 }
 
 static void update_wanted_crc(const struct test_mode *t, igt_crc_t *crc)
@@ -2674,7 +2670,7 @@ static void modesetfrombusy_subtest(const struct test_mode *t)
 	params->primary.fb = &fb2;
 	set_mode_for_params(params);
 
-	do_assertions(0);
+	do_assertions(ASSERT_NO_IDLE_GPU);
 
 	stop_busy_thread();
 
@@ -2841,7 +2837,7 @@ static void badstride_subtest(const struct test_mode *t)
 
 	create_fb(t->format, params->primary.fb->width + 4096, params->primary.fb->height,
 		  opt.tiling, t->plane, &wide_fb);
-	igt_assert(wide_fb.stride > 16384);
+	igt_assert(wide_fb.strides[0] > 16384);
 
 	fill_fb(&wide_fb, COLOR_PRIM_BG);
 
@@ -2898,7 +2894,7 @@ static void badstride_subtest(const struct test_mode *t)
  */
 static void stridechange_subtest(const struct test_mode *t)
 {
-	struct igt_fb new_fb, *old_fb;
+	struct igt_fb *new_fb, *old_fb;
 	struct modeset_params *params = pick_params(t);
 	int rc;
 
@@ -2906,17 +2902,16 @@ static void stridechange_subtest(const struct test_mode *t)
 
 	old_fb = params->primary.fb;
 
-	create_fb(t->format, params->primary.fb->width + 512, params->primary.fb->height,
-		  opt.tiling, t->plane, &new_fb);
-	fill_fb(&new_fb, COLOR_PRIM_BG);
-
-	igt_assert(old_fb->stride != new_fb.stride);
-
 	/* We can't assert that FBC will be enabled since there may not be
 	 * enough space for the CFB, but we can check the CRC. */
-	params->primary.fb = &new_fb;
+	new_fb = &fbs[t->format].big;
+	igt_assert(old_fb->strides[0] != new_fb->strides[0]);
+
+	params->primary.fb = new_fb;
+	fill_fb_region(&params->primary, COLOR_PRIM_BG);
+
 	set_mode_for_params(params);
-	do_assertions(DONT_ASSERT_FEATURE_STATUS);
+	do_assertions(DONT_ASSERT_FBC_STATUS);
 
 	/* Go back to the fb that can have FBC. */
 	params->primary.fb = old_fb;
@@ -2924,9 +2919,9 @@ static void stridechange_subtest(const struct test_mode *t)
 	do_assertions(0);
 
 	/* This operation is the same as above, but with the planes API. */
-	params->primary.fb = &new_fb;
+	params->primary.fb = new_fb;
 	set_prim_plane_for_params(params);
-	do_assertions(DONT_ASSERT_FEATURE_STATUS);
+	do_assertions(DONT_ASSERT_FBC_STATUS);
 
 	params->primary.fb = old_fb;
 	set_prim_plane_for_params(params);
@@ -2936,11 +2931,9 @@ static void stridechange_subtest(const struct test_mode *t)
 	 * Try to set a new stride. with the page flip api. This is allowed
 	 * with the atomic page flip helper, but not with the legacy page flip.
 	 */
-	rc = drmModePageFlip(drm.fd, drm.display.pipes[params->pipe].crtc_id, new_fb.fb_id, 0, NULL);
+	rc = drmModePageFlip(drm.fd, drm.display.pipes[params->pipe].crtc_id, new_fb->fb_id, 0, NULL);
 	igt_assert(rc == -EINVAL || rc == 0);
-	do_assertions(0);
-
-	igt_remove_fb(drm.fd, &new_fb);
+	do_assertions(rc ? 0 : DONT_ASSERT_FBC_STATUS);
 }
 
 /**
@@ -3077,19 +3070,23 @@ static int opt_handler(int option, int option_index, void *data)
 	case 'x':
 		errno = 0;
 		opt.shared_fb_x_offset = strtol(optarg, NULL, 0);
-		igt_assert(errno == 0);
+		if (errno != 0)
+			return IGT_OPT_HANDLER_ERROR;
 		break;
 	case 'y':
 		errno = 0;
 		opt.shared_fb_y_offset = strtol(optarg, NULL, 0);
-		igt_assert(errno == 0);
+		if (errno != 0)
+			return IGT_OPT_HANDLER_ERROR;
 		break;
 	case '1':
-		igt_assert_eq(opt.only_pipes, PIPE_COUNT);
+		if (opt.only_pipes != PIPE_COUNT)
+			return IGT_OPT_HANDLER_ERROR;
 		opt.only_pipes = PIPE_SINGLE;
 		break;
 	case '2':
-		igt_assert_eq(opt.only_pipes, PIPE_COUNT);
+		if (opt.only_pipes != PIPE_COUNT)
+			return IGT_OPT_HANDLER_ERROR;
 		opt.only_pipes = PIPE_DUAL;
 		break;
 	case 'l':
@@ -3097,14 +3094,16 @@ static int opt_handler(int option, int option_index, void *data)
 			opt.tiling = LOCAL_I915_FORMAT_MOD_X_TILED;
 		else if (!strcmp(optarg, "y"))
 			opt.tiling = LOCAL_I915_FORMAT_MOD_Y_TILED;
-		else
-			igt_assert_f(false, "Bad tiling value: %s\n", optarg);
+		else {
+			igt_warn("Bad tiling value: %s\n", optarg);
+			return IGT_OPT_HANDLER_ERROR;
+		}
 		break;
 	default:
-		igt_assert(false);
+		return IGT_OPT_HANDLER_ERROR;
 	}
 
-	return 0;
+	return IGT_OPT_HANDLER_SUCCESS;
 }
 
 const char *help_str =
@@ -3252,28 +3251,26 @@ static const char *flip_str(enum flip_type flip)
 
 #define TEST_MODE_ITER_END } } } } } }
 
-int main(int argc, char *argv[])
+struct option long_options[] = {
+	{ "no-status-check",          0, 0, 's'},
+	{ "no-crc-check",             0, 0, 'c'},
+	{ "no-fbc-compression-check", 0, 0, 'o'},
+	{ "no-fbc-action-check",      0, 0, 'a'},
+	{ "no-edp",                   0, 0, 'e'},
+	{ "use-small-modes",          0, 0, 'm'},
+	{ "show-hidden",              0, 0, 'i'},
+	{ "step",                     0, 0, 't'},
+	{ "shared-fb-x",              1, 0, 'x'},
+	{ "shared-fb-y",              1, 0, 'y'},
+	{ "1p-only",                  0, 0, '1'},
+	{ "2p-only",                  0, 0, '2'},
+	{ "tiling",                   1, 0, 'l'},
+	{ 0, 0, 0, 0 }
+};
+
+igt_main_args("", long_options, help_str, opt_handler, NULL)
 {
 	struct test_mode t;
-	struct option long_options[] = {
-		{ "no-status-check",          0, 0, 's'},
-		{ "no-crc-check",             0, 0, 'c'},
-		{ "no-fbc-compression-check", 0, 0, 'o'},
-		{ "no-fbc-action-check",      0, 0, 'a'},
-		{ "no-edp",                   0, 0, 'e'},
-		{ "use-small-modes",          0, 0, 'm'},
-		{ "show-hidden",              0, 0, 'i'},
-		{ "step",                     0, 0, 't'},
-		{ "shared-fb-x",              1, 0, 'x'},
-		{ "shared-fb-y",              1, 0, 'y'},
-		{ "1p-only",                  0, 0, '1'},
-		{ "2p-only",                  0, 0, '2'},
-		{ "tiling",                   1, 0, 'l'},
-		{ 0, 0, 0, 0 }
-	};
-
-	igt_subtest_init_parse_opts(&argc, argv, "", long_options, help_str,
-				    opt_handler, NULL);
 
 	igt_fixture
 		setup_environment();
@@ -3480,6 +3477,4 @@ int main(int argc, char *argv[])
 
 	igt_fixture
 		teardown_environment();
-
-	igt_exit();
 }
