@@ -41,6 +41,7 @@ typedef struct {
 	int drm_fd;
 	igt_display_t display;
 	igt_pipe_crc_t *pipe_crc;
+	uint32_t crop;
 } data_t;
 
 static color_t red   = { 1.0f, 0.0f, 0.0f };
@@ -218,28 +219,23 @@ test_plane_position_with_output(data_t *data,
 static void
 test_plane_position(data_t *data, enum pipe pipe, unsigned int flags)
 {
+	int n_planes = data->display.pipes[pipe].n_planes;
 	igt_output_t *output;
-	int connected_outs = 0;
+	igt_crc_t reference_crc;
 
-	for_each_valid_output_on_pipe(&data->display, pipe, output) {
-		int n_planes = data->display.pipes[pipe].n_planes;
-		igt_crc_t reference_crc;
+	output = igt_get_single_output_for_pipe(&data->display, pipe);
+	igt_require(output);
 
-		test_init(data, pipe);
+	test_init(data, pipe);
 
-		test_grab_crc(data, output, pipe, &green, &reference_crc);
+	test_grab_crc(data, output, pipe, &green, &reference_crc);
 
-		for (int plane = 1; plane < n_planes; plane++)
-			test_plane_position_with_output(data, pipe, plane,
-							output, &reference_crc,
-							flags);
+	for (int plane = 1; plane < n_planes; plane++)
+		test_plane_position_with_output(data, pipe, plane,
+						output, &reference_crc,
+						flags);
 
-		test_fini(data);
-
-		connected_outs++;
-	}
-
-	igt_skip_on(connected_outs == 0);
+	test_fini(data);
 }
 
 /*
@@ -339,31 +335,24 @@ test_plane_panning_with_output(data_t *data,
 static void
 test_plane_panning(data_t *data, enum pipe pipe, unsigned int flags)
 {
+	int n_planes = data->display.pipes[pipe].n_planes;
 	igt_output_t *output;
-	int connected_outs = 0;
+	igt_crc_t red_crc;
+	igt_crc_t blue_crc;
 
-	for_each_valid_output_on_pipe(&data->display, pipe, output) {
-		int n_planes = data->display.pipes[pipe].n_planes;
-		igt_crc_t red_crc;
-		igt_crc_t blue_crc;
+	output = igt_get_single_output_for_pipe(&data->display, pipe);
+	igt_require(output);
 
-		test_init(data, pipe);
+	test_init(data, pipe);
 
-		test_grab_crc(data, output, pipe, &red, &red_crc);
-		test_grab_crc(data, output, pipe, &blue, &blue_crc);
+	test_grab_crc(data, output, pipe, &red, &red_crc);
+	test_grab_crc(data, output, pipe, &blue, &blue_crc);
 
-		for (int plane = 1; plane < n_planes; plane++)
-			test_plane_panning_with_output(data, pipe, plane,
-						       output,
-						       &red_crc, &blue_crc,
-						       flags);
+	for (int plane = 1; plane < n_planes; plane++)
+		test_plane_panning_with_output(data, pipe, plane, output,
+					       &red_crc, &blue_crc, flags);
 
-		test_fini(data);
-
-		connected_outs++;
-	}
-
-	igt_skip_on(connected_outs == 0);
+	test_fini(data);
 }
 
 static const color_t colors[] = {
@@ -400,64 +389,265 @@ static void set_legacy_lut(data_t *data, enum pipe pipe,
 	free(lut);
 }
 
-#define IGT_FORMAT_FMT "%c%c%c%c (0x%08x)"
-#define IGT_FORMAT_ARGS(f) ((f) >> 0) & 0xff, ((f) >> 8) & 0xff, \
-		((f) >> 16) & 0xff, ((f) >> 24) & 0xff, (f)
+static bool set_c8_legacy_lut(data_t *data, enum pipe pipe,
+			      uint16_t mask)
+{
+	igt_pipe_t *pipe_obj = &data->display.pipes[pipe];
+	drmModeCrtc *crtc;
+	uint16_t *r, *g, *b;
+	int i, lut_size;
+
+	crtc = drmModeGetCrtc(data->drm_fd, pipe_obj->crtc_id);
+	lut_size = crtc->gamma_size;
+	drmModeFreeCrtc(crtc);
+
+	if (lut_size != 256)
+		return false;
+
+	r = malloc(sizeof(uint16_t) * 3 * lut_size);
+	g = r + lut_size;
+	b = g + lut_size;
+
+	/* igt_fb uses RGB332 for C8 */
+	for (i = 0; i < lut_size; i++) {
+		r[i] = (((i & 0xe0) >> 5) * 0xffff / 0x7) & mask;
+		g[i] = (((i & 0x1c) >> 2) * 0xffff / 0x7) & mask;
+		b[i] = (((i & 0x03) >> 0) * 0xffff / 0x3) & mask;
+	}
+
+	igt_assert_eq(drmModeCrtcSetGamma(data->drm_fd, pipe_obj->crtc_id,
+					  lut_size, r, g, b), 0);
+
+	free(r);
+
+	return true;
+}
 
 static void test_format_plane_color(data_t *data, enum pipe pipe,
 				    igt_plane_t *plane,
-				    uint32_t format, int width, int height,
+				    uint32_t format, uint64_t modifier,
+				    int width, int height,
 				    int color, igt_crc_t *crc, struct igt_fb *fb)
 {
 	const color_t *c = &colors[color];
 	struct igt_fb old_fb = *fb;
 
-	igt_create_color_fb(data->drm_fd, width, height,
-			    format, LOCAL_DRM_FORMAT_MOD_NONE,
-			    c->red, c->green, c->blue, fb);
+	if (data->crop == 0 || format == DRM_FORMAT_XRGB8888) {
+		igt_create_color_fb(data->drm_fd, width, height,
+				    format, modifier,
+				    c->red, c->green, c->blue, fb);
+	} else {
+		/*
+		 * paint border in inverted color, then visible area in middle
+		 * with correct color for clamping test
+		 */
+		cairo_t *cr;
+
+		igt_create_fb(data->drm_fd,
+			      width + data->crop * 2,
+			      height + data->crop * 2,
+			      format, modifier, fb);
+
+		cr = igt_get_cairo_ctx(data->drm_fd, fb);
+
+		igt_paint_color(cr, 0, 0,
+				width + data->crop * 2,
+				height + data->crop * 2,
+				1.0f - c->red,
+				1.0f - c->green,
+				1.0f - c->blue);
+
+		igt_paint_color(cr, data->crop, data->crop,
+				width, height,
+				c->red, c->green, c->blue);
+
+		igt_put_cairo_ctx(data->drm_fd, fb, cr);
+	}
 
 	igt_plane_set_fb(plane, fb);
 
-	igt_display_commit2(&data->display, data->display.is_atomic ? COMMIT_ATOMIC : COMMIT_UNIVERSAL);
+	/*
+	 * if clamping test. DRM_FORMAT_XRGB8888 is used for reference color.
+	 */
+	if (data->crop != 0 && format != DRM_FORMAT_XRGB8888) {
+		igt_fb_set_position(fb, plane, data->crop, data->crop);
+		igt_fb_set_size(fb, plane, width, height);
+		igt_plane_set_size(plane, width, height);
+	}
 
-	/* make sure the crc we get is for the new fb */
-	igt_wait_for_vblank(data->drm_fd, pipe);
-	igt_pipe_crc_drain(data->pipe_crc);
-	igt_pipe_crc_get_single(data->pipe_crc, crc);
+	igt_display_commit2(&data->display, data->display.is_atomic ? COMMIT_ATOMIC : COMMIT_UNIVERSAL);
+	igt_pipe_crc_get_current(data->display.drm_fd, data->pipe_crc, crc);
 
 	igt_remove_fb(data->drm_fd, &old_fb);
 }
 
-static void test_format_plane(data_t *data, enum pipe pipe,
+static int num_unique_crcs(const igt_crc_t crc[], int num_crc)
+{
+	int num_unique_crc = 0;
+
+	for (int i = 0; i < num_crc; i++) {
+		int j;
+
+		for (j = i + 1; j < num_crc; j++) {
+			if (igt_check_crc_equal(&crc[i], &crc[j]))
+				break;
+		}
+
+		if (j == num_crc)
+			num_unique_crc++;
+	}
+
+	return num_unique_crc;
+}
+
+static bool test_format_plane(data_t *data, enum pipe pipe,
 			      igt_output_t *output, igt_plane_t *plane)
 {
-	igt_plane_t *primary;
-	struct igt_fb primary_fb;
 	struct igt_fb fb = {};
 	drmModeModeInfo *mode;
 	uint32_t format, ref_format;
+	uint64_t modifier, ref_modifier;
 	uint64_t width, height;
 	igt_crc_t ref_crc[ARRAY_SIZE(colors)];
+	bool result = true;
+
+	/*
+	 * No clamping test for cursor plane
+	 */
+	if (data->crop != 0 && plane->type == DRM_PLANE_TYPE_CURSOR)
+		return true;
 
 	mode = igt_output_get_mode(output);
 	if (plane->type != DRM_PLANE_TYPE_CURSOR) {
 		width = mode->hdisplay;
 		height = mode->vdisplay;
 		ref_format = format = DRM_FORMAT_XRGB8888;
+		ref_modifier = modifier = DRM_FORMAT_MOD_NONE;
 	} else {
 		if (!plane->drm_plane) {
 			igt_debug("Only legacy cursor ioctl supported, skipping cursor plane\n");
-			return;
+			return true;
 		}
 		do_or_die(drmGetCap(data->drm_fd, DRM_CAP_CURSOR_WIDTH, &width));
 		do_or_die(drmGetCap(data->drm_fd, DRM_CAP_CURSOR_HEIGHT, &height));
 		ref_format = format = DRM_FORMAT_ARGB8888;
+		ref_modifier = modifier = DRM_FORMAT_MOD_NONE;
 	}
 
 	igt_debug("Testing connector %s on %s plane %s.%u\n",
 		  igt_output_name(output), kmstest_plane_type_name(plane->type),
 		  kmstest_pipe_name(pipe), plane->index);
 
+	igt_pipe_crc_start(data->pipe_crc);
+
+	igt_info("Testing format " IGT_FORMAT_FMT " / modifier 0x%" PRIx64 " on %s.%u\n",
+		 IGT_FORMAT_ARGS(format), modifier,
+		 kmstest_pipe_name(pipe), plane->index);
+
+	if (data->display.is_atomic) {
+		struct igt_fb test_fb;
+		int ret;
+
+		igt_create_fb(data->drm_fd, 64, 64, format,
+			      LOCAL_DRM_FORMAT_MOD_NONE, &test_fb);
+
+		igt_plane_set_fb(plane, &test_fb);
+
+		ret = igt_display_try_commit_atomic(&data->display, DRM_MODE_ATOMIC_TEST_ONLY, NULL);
+
+		if (!ret) {
+			width = test_fb.width;
+			height = test_fb.height;
+		}
+
+		igt_plane_set_fb(plane, NULL);
+
+		igt_remove_fb(data->drm_fd, &test_fb);
+	}
+
+	for (int i = 0; i < ARRAY_SIZE(colors); i++) {
+		test_format_plane_color(data, pipe, plane,
+					format, modifier,
+					width, height,
+					i, &ref_crc[i], &fb);
+	}
+
+	/*
+	 * Make sure we have some difference between the colors. This
+	 * at least avoids claiming success when everything is just
+	 * black all the time (eg. if the plane is never even on).
+	 */
+	igt_require(num_unique_crcs(ref_crc, ARRAY_SIZE(colors)) > 1);
+
+	for (int i = 0; i < plane->format_mod_count; i++) {
+		int crc_mismatch_count = 0;
+		int crc_mismatch_mask = 0;
+		igt_crc_t crc;
+
+		format = plane->formats[i];
+		modifier = plane->modifiers[i];
+
+		if (format == ref_format &&
+		    modifier == ref_modifier)
+			continue;
+
+		if (format == DRM_FORMAT_C8) {
+			if (!set_c8_legacy_lut(data, pipe, 0xfc00))
+				continue;
+		} else {
+			if (!igt_fb_supported_format(format))
+				continue;
+		}
+
+		igt_info("Testing format " IGT_FORMAT_FMT " / modifier 0x%" PRIx64 " on %s.%u\n",
+			 IGT_FORMAT_ARGS(format), modifier,
+			 kmstest_pipe_name(pipe), plane->index);
+
+		for (int j = 0; j < ARRAY_SIZE(colors); j++) {
+			test_format_plane_color(data, pipe, plane,
+						format, modifier,
+						width, height,
+						j, &crc, &fb);
+
+			if (!igt_check_crc_equal(&crc, &ref_crc[j])) {
+				crc_mismatch_count++;
+				crc_mismatch_mask |= (1 << j);
+				result = false;
+			}
+		}
+
+		if (format == DRM_FORMAT_C8)
+			set_legacy_lut(data, pipe, 0xfc00);
+
+		if (crc_mismatch_count)
+			igt_warn("CRC mismatches with format " IGT_FORMAT_FMT " on %s.%u with %d/%d solid colors tested (0x%X)\n",
+				 IGT_FORMAT_ARGS(format), kmstest_pipe_name(pipe),
+				 plane->index, crc_mismatch_count, (int)ARRAY_SIZE(colors), crc_mismatch_mask);
+	}
+
+	igt_pipe_crc_stop(data->pipe_crc);
+
+	igt_plane_set_fb(plane, NULL);
+	igt_remove_fb(data->drm_fd, &fb);
+
+	return result;
+}
+
+static void
+test_pixel_formats(data_t *data, enum pipe pipe)
+{
+	struct igt_fb primary_fb;
+	igt_plane_t *primary;
+	drmModeModeInfo *mode;
+	bool result;
+	igt_output_t *output;
+	igt_plane_t *plane;
+
+	output = igt_get_single_output_for_pipe(&data->display, pipe);
+	igt_require(output);
+
+	mode = igt_output_get_mode(output);
+
 	igt_create_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 		      DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE, &primary_fb);
 
@@ -470,70 +660,22 @@ static void test_format_plane(data_t *data, enum pipe pipe,
 	set_legacy_lut(data, pipe, 0xfc00);
 
 	test_init(data, pipe);
-	igt_pipe_crc_start(data->pipe_crc);
 
-	igt_info("Testing format " IGT_FORMAT_FMT " on %s.%u\n",
-		 IGT_FORMAT_ARGS(format),
-		 kmstest_pipe_name(pipe), plane->index);
+	result = true;
+	for_each_plane_on_pipe(&data->display, pipe, plane)
+		result &= test_format_plane(data, pipe, output, plane);
 
-	for (int i = 0; i < ARRAY_SIZE(colors); i++) {
-		test_format_plane_color(data, pipe, plane,
-					format, width, height,
-					i, &ref_crc[i], &fb);
-	}
-
-	for (int i = 0; i < plane->drm_plane->count_formats; i++) {
-		igt_crc_t crc;
-
-		format = plane->drm_plane->formats[i];
-
-		if (format == ref_format)
-			continue;
-
-		if (!igt_fb_supported_format(format))
-			continue;
-
-		igt_info("Testing format " IGT_FORMAT_FMT " on %s.%u\n",
-			 IGT_FORMAT_ARGS(format),
-			 kmstest_pipe_name(pipe), plane->index);
-
-		for (int j = 0; j < ARRAY_SIZE(colors); j++) {
-			test_format_plane_color(data, pipe, plane,
-						format, width, height,
-						j, &crc, &fb);
-
-			igt_assert_crc_equal(&crc, &ref_crc[j]);
-		}
-	}
-
-	igt_pipe_crc_stop(data->pipe_crc);
 	test_fini(data);
 
 	set_legacy_lut(data, pipe, 0xffff);
 
 	igt_plane_set_fb(primary, NULL);
-	igt_plane_set_fb(plane, NULL);
+	igt_output_set_pipe(output, PIPE_NONE);
 	igt_display_commit2(&data->display, data->display.is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
 
-	igt_remove_fb(data->drm_fd, &fb);
 	igt_remove_fb(data->drm_fd, &primary_fb);
-}
 
-static void
-test_pixel_formats(data_t *data, enum pipe pipe)
-{
-	igt_output_t *output;
-
-	igt_display_require_output_on_pipe(&data->display, pipe);
-
-	for_each_valid_output_on_pipe(&data->display, pipe, output) {
-		igt_plane_t *plane;
-
-		for_each_plane_on_pipe(&data->display, pipe, plane)
-			test_format_plane(data, pipe, output, plane);
-
-		igt_output_set_pipe(output, PIPE_ANY);
-	}
+	igt_assert_f(result, "At least one CRC mismatch happened\n");
 }
 
 static void
@@ -548,6 +690,13 @@ run_tests_for_pipe_plane(data_t *data, enum pipe pipe)
 		      kmstest_pipe_name(pipe))
 		test_pixel_formats(data, pipe);
 
+	igt_subtest_f("pixel-format-pipe-%s-planes-source-clamping",
+		      kmstest_pipe_name(pipe)) {
+		data->crop = 4;
+		test_pixel_formats(data, pipe);
+	}
+
+	data->crop = 0;
 	igt_subtest_f("plane-position-covered-pipe-%s-planes",
 		      kmstest_pipe_name(pipe))
 		test_plane_position(data, pipe, TEST_POSITION_FULLY_COVERED);
@@ -584,12 +733,12 @@ igt_main
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		data.drm_fd = drm_open_driver_master(DRIVER_INTEL);
+		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
 
 		kmstest_set_vt_graphics_mode();
 
 		igt_require_pipe_crc(data.drm_fd);
-		igt_display_init(&data.display, data.drm_fd);
+		igt_display_require(&data.display, data.drm_fd);
 	}
 
 	for_each_pipe_static(pipe)
