@@ -228,7 +228,7 @@ static void dump_backlight_info(struct context *context,
 				const struct bdb_block *block)
 {
 	const struct bdb_lfp_backlight_data *backlight = block->data;
-	const struct bdb_lfp_backlight_data_entry *blc;
+	const struct lfp_backlight_data_entry *blc;
 
 	if (sizeof(*blc) != backlight->entry_size) {
 		printf("\tBacklight struct sizes don't match (expected %zu, got %u), skipping\n",
@@ -399,6 +399,26 @@ static const char *mipi_bridge_type(uint8_t type)
 	}
 }
 
+static void dump_hmdi_max_data_rate(uint8_t hdmi_max_data_rate)
+{
+	static const uint16_t max_data_rate[] = {
+		[HDMI_MAX_DATA_RATE_PLATFORM] = 0,
+		[HDMI_MAX_DATA_RATE_297] = 297,
+		[HDMI_MAX_DATA_RATE_165] = 165,
+	};
+
+	if (hdmi_max_data_rate >= ARRAY_SIZE(max_data_rate))
+		printf("\t\tHDMI max data rate: <unknown> (0x%02x)\n",
+		       hdmi_max_data_rate);
+	else if (hdmi_max_data_rate == HDMI_MAX_DATA_RATE_PLATFORM)
+		printf("\t\tHDMI max data rate: <platform max> (0x%02x)\n",
+		       hdmi_max_data_rate);
+	else
+		printf("\t\tHDMI max data rate: %d MHz (0x%02x)\n",
+		       max_data_rate[hdmi_max_data_rate],
+		       hdmi_max_data_rate);
+}
+
 static void dump_child_device(struct context *context,
 			      const struct child_device_config *child)
 {
@@ -419,7 +439,7 @@ static void dump_child_device(struct context *context,
 		printf("\t\tDP onboard redriver: 0x%02x\n", child->dp_onboard_redriver);
 		printf("\t\tDP ondock redriver: 0x%02x\n", child->dp_ondock_redriver);
 		printf("\t\tHDMI level shifter value: 0x%02x\n", child->hdmi_level_shifter_value);
-		printf("\t\tHDMI max data rate: 0x%02x\n", child->hdmi_max_data_rate);
+		dump_hmdi_max_data_rate(child->hdmi_max_data_rate);
 		printf("\t\tOffset to DTD buffer for edidless CHILD: 0x%02x\n", child->dtd_buf_ptr);
 		printf("\t\tEdidless EFP: %s\n", YESNO(child->edidless_efp));
 		printf("\t\tCompression enable: %s\n", YESNO(child->compression_enable));
@@ -609,8 +629,8 @@ static void dump_lvds_data(struct context *context,
 		const uint8_t *lfp_data_ptr =
 		    (const uint8_t *) lvds_data->data + lfp_data_size * i;
 		const uint8_t *timing_data = lfp_data_ptr + dvo_offset;
-		const struct bdb_lvds_lfp_data_entry *lfp_data =
-		    (const struct bdb_lvds_lfp_data_entry *)lfp_data_ptr;
+		const struct lvds_lfp_data_entry *lfp_data =
+		    (const struct lvds_lfp_data_entry *)lfp_data_ptr;
 		char marker;
 
 		if (i != context->panel_type && !context->dump_all_panel_types)
@@ -1049,6 +1069,7 @@ static void dump_mipi_config(struct context *context,
 
 	printf("\tGeneral Param\n");
 	printf("\t\t BTA disable: %s\n", config->bta ? "Disabled" : "Enabled");
+	printf("\t\t Panel Rotation: %d degrees\n", config->rotation * 90);
 
 	printf("\t\t Video Mode Color Format: ");
 	if (config->videomode_color_format == 0)
@@ -1130,7 +1151,7 @@ static void dump_mipi_config(struct context *context,
 	printf("\t\tPanel power cycle delay: %d\n", pps->panel_power_cycle_delay);
 }
 
-static const uint8_t *mipi_dump_send_packet(const uint8_t *data)
+static const uint8_t *mipi_dump_send_packet(const uint8_t *data, uint8_t seq_version)
 {
 	uint8_t flags, type;
 	uint16_t len, i;
@@ -1153,29 +1174,36 @@ static const uint8_t *mipi_dump_send_packet(const uint8_t *data)
 	return data;
 }
 
-static const uint8_t *mipi_dump_delay(const uint8_t *data)
+static const uint8_t *mipi_dump_delay(const uint8_t *data, uint8_t seq_version)
 {
 	printf("\t\tDelay: %u us\n", *((const uint32_t *)data));
 
 	return data + 4;
 }
 
-static const uint8_t *mipi_dump_gpio(const uint8_t *data)
+static const uint8_t *mipi_dump_gpio(const uint8_t *data, uint8_t seq_version)
 {
-	uint8_t index, flags;
+	uint8_t index, number, flags;
 
-	index = *data++;
-	flags = *data++;
+	if (seq_version >= 3) {
+		index = *data++;
+		number = *data++;
+		flags = *data++;
 
-	printf("\t\tGPIO index %u, source %d, set %d\n",
-	       index,
-	       (flags >> 1) & 3,
-	       flags & 1);
+		printf("\t\tGPIO index %u, number %u, set %d (0x%02x)\n",
+		       index, number, flags & 1, flags);
+	} else {
+		index = *data++;
+		flags = *data++;
+
+		printf("\t\tGPIO index %u, source %d, set %d (0x%02x)\n",
+		       index, (flags >> 1) & 3, flags & 1, flags);
+	}
 
 	return data;
 }
 
-static const uint8_t *mipi_dump_i2c(const uint8_t *data)
+static const uint8_t *mipi_dump_i2c(const uint8_t *data, uint8_t seq_version)
 {
 	uint8_t flags, index, bus, offset, len, i;
 	uint16_t address;
@@ -1197,7 +1225,7 @@ static const uint8_t *mipi_dump_i2c(const uint8_t *data)
 	return data;
 }
 
-typedef const uint8_t * (*fn_mipi_elem_dump)(const uint8_t *data);
+typedef const uint8_t * (*fn_mipi_elem_dump)(const uint8_t *data, uint8_t seq_version);
 
 static const fn_mipi_elem_dump dump_elem[] = {
 	[MIPI_SEQ_ELEM_SEND_PKT] = mipi_dump_send_packet,
@@ -1258,7 +1286,13 @@ static const uint8_t *dump_sequence(const uint8_t *data, uint8_t seq_version)
 			operation_size = *data++;
 
 		if (mipi_elem_dump) {
-			data = mipi_elem_dump(data);
+			const uint8_t *next = data + operation_size;
+
+			data = mipi_elem_dump(data, seq_version);
+
+			if (operation_size && next != data)
+				printf("Error: Inconsistent operation size: %d\n",
+					operation_size);
 		} else if (operation_size) {
 			/* We have size, skip. */
 			data += operation_size;
